
import sys
import os
import re

from optparse import OptionParser, IndentedHelpFormatter

import plugintools
import checkenvvars
import version
import info
import term
import utils

USAGE = "%prog [OPTIONS] -- <COMMAND> [<ARG1> [<ARG2> ...]]"
VERSION ="VHISTIFY_VERSION"
DESCRIPTION = ""


def _printPlugins(defaultPluginNames):
    columns = term.getNumColumns()
    
    print "This is a list of all installed plugins:\n"
    lines = plugintools.createPluginList(columns - 5, defaultPluginNames)
    for line in lines:
        print line
    sys.exit(0)


def _printPluginHelp(pluginname):
    columns = term.getNumColumns()
    
    lines = plugintools.getPluginHelp(pluginname, columns - 5)
    print "Documentation of plugin %s:" % pluginname
    for line in lines:
        print line
    sys.exit(0)


def _readEnvironmentVariables():
    plugins = os.environ.get("VHISTIFY_PLUGINS", None)
    defaultPlugins = os.environ.get("VHISTIFY_DEFAULT_PLUGINS", None)
    
    renameOldVhistFiles = os.environ.get("VHISTIFY_RENAME_OLD_VHISTFILES", None)
    if renameOldVhistFiles is not None:
        if renameOldVhistFiles.lower() in ["0", "false", "no", ""]:
            renameOldVhistFiles = False
        else:
            renameOldVhistFiles = True
    
    title = os.environ.get("VHISTIFY_TITLE", None)
    description = os.environ.get("VHISTIFY_DESCRIPTION", None)
    comment = os.environ.get("VHISTIFY_COMMENT", None)
    
    infoObj = info.Info()
    infoObj.pluginNames = plugintools.splitPluginList(plugins)
    infoObj.defaultPluginNames = plugintools.splitPluginList(defaultPlugins)
    infoObj.title = title
    infoObj.description = description
    infoObj.comment = comment
    infoObj.renameOldVhistFiles = renameOldVhistFiles
    
    return infoObj


def _parseCommandline(options, args, infoObj):
    parser = OptionParser(usage=USAGE, add_help_option=False)
    parser.add_option("--help", dest="help", default=False,
            action="store_true", help="show this help message and exit")
    parser.add_option("--plugin-help", dest="helpPlugin", default=None,
            metavar="PLUGINNAME",
            help="show the help text for the given plugin and exit")
    parser.add_option("--version", dest="version", default=False,
            action="store_true",
            help="show program's version number and exit")
    parser.add_option("--title", dest="title", default=None,
            help="title of the vhist section", metavar="TITLE")
    parser.add_option("--description", dest="description", default=None,
            help="description of the vhist section", metavar="DESCRIPTION")
    parser.add_option("--comment", dest="comment", default=None,
            help="additional comment about the vhist section",
            metavar="COMMENT")
    parser.add_option("--plugins", dest="plugins", default=None,
            help="comma-separated list of plugins", metavar="PLUGINS")
    parser.add_option("--list-plugins", dest="listPlugins", default=False,
            action="store_true",
            help="shows a list of all installed plugins and quits")
    parser.add_option("--rename-old-vhistfiles", dest="renameOldVhistFiles",
            default=None, action="store_true",
            help="if enabled, existing old vhist files will be renamed "
            "before new vhist files are created.")
    parser.add_option("--no-stdin", dest="noStdin", default=False,
            action="store_true",
            help="if set, the vhistified process will not receive input "
            "from stdin.")
    parser.add_option("--debug", dest="debug", default=False,
            action="store_true", help="in the case of an error, shows a "
            "more elaborate error message. Will also pause vhistify after "
            "vhistify generated the VHIST files so that you can inspect "
            "vhistify's log files")
    
    options, wrongArgs = parser.parse_args(options)
    if len(wrongArgs) > 0:
        parser.error("internal error while parsing commandline arguments. "
                "Found arguments %s even though non were expected." %
                wrongArgs)
    
    # fill info object
    infoObj.args = args
    if options.plugins is not None:
        infoObj.pluginNames = plugintools.splitPluginList(options.plugins)
    
    if options.title is not None:
        infoObj.title = options.title
    
    if options.description is not None:
        infoObj.description = options.description
    
    if options.comment is not None:
        infoObj.comment = options.comment
    
    if options.renameOldVhistFiles is not None:
        infoObj.renameOldVhistFiles = options.renameOldVhistFiles
    
    if options.noStdin is not None:
        infoObj.noStdin = options.noStdin
    
    if options.debug is not None:
        infoObj.debug = options.debug
    
    # options and parser are later used to print help messages
    return infoObj, options, parser


def _handleHelpOptions(options, parser, defaultPluginNames):
    # handle help/version options
    if options.help:
        # make sure that the help text has the same width as the terminal
        parser.formatter = IndentedHelpFormatter(width=term.getNumColumns())
        parser.print_help()
        parser.exit()
    
    if options.helpPlugin != None:
        _printPluginHelp(options.helpPlugin)
    
    if options.version:
        parser.version = "%prog " + version.getVersion()
        parser.print_version()
        parser.exit()
    
    if options.listPlugins:
        _printPlugins(defaultPluginNames)


def _updateAttribute(infoObj, attr, newValue):
    if not hasattr(infoObj, attr) or getattr(infoObj, attr) is None:
        setattr(infoObj, attr, newValue)


def _addDefaults(infoObj):
    _updateAttribute(infoObj, "pluginNames", [])
    _updateAttribute(infoObj, "defaultPluginNames",
            plugintools.getDefaultPlugins())
    _updateAttribute(infoObj, "renameOldVhistFiles", False)
    
    return infoObj


def _preparseCommandline(args):
    """ Detect the point in the commandline, at which the vhistify options
    end and the arguments (command to execute plus it's options/parameters)
    begins. This routine makes it possible to distinguish between options
    of vhistify and options of the command to execute without using -- in
    the commandline """
    
    # all options with their number of parameters (all options must have
    # zero or one parameters)
    options = {"--help": 0, "--plugin-help": 1, "--version" : 0,
            "--title": 1, "--description": 1, "--comment": 1,
            "--plugins": 1, "--list-plugins": 0,
            "--rename-old-vhistfiles": 0, "--no-stdin": 0, "--debug": 0}
    
    
    skipNext = False # True if next argument is the parameter of an option
    splitPoint = len(args) # all entries are options, no arguments
    
    for i, arg in enumerate(args):
        if skipNext: # this argument is the value of the previous option
            skipNext = False
            continue
        
        # options are either --debug, --title=foo or --title (parameter stored
        # in the next argument)
        found = re.match("(?P<arg>--[-a-z]*)(=(?P<value>.*))?", arg)
        if found != None: # possibly valid option
            argname = found.group("arg")
            argvalue = found.group("value")
            if argname in options: # valid option
                if options[argname] == 0 or argvalue != None:
                    continue
                else:
                    skipNext = True
                    continue
            elif arg == "--": # end of options, next arg in first argument
                splitPoint = i + 1
                break
            else: # not a valid option, first argument
                splitPoint = i
                break
        else: # not a valid option, first argument
            splitPoint = i
            break
    
    options = args[:splitPoint]
    args = args[splitPoint:]
    
    if len(args) > 0 and args[0].startswith("-"):
        print ("*** WARNING: the program to execute (%s) starts with a '-'."
                " Maybe it is a misspelled option.") % args[0]
    return options, args


def getOptions(inputArgs):
    checkenvvars.checkEnvironmentVariables()
    infoObj = _readEnvironmentVariables()
    options, args = _preparseCommandline(inputArgs[1:])
    infoObj, options, parser = _parseCommandline(options, args, infoObj)
    infoObj.originalArgs = inputArgs
    infoObj = _addDefaults(infoObj)
    
    _handleHelpOptions(options, parser, infoObj.defaultPluginNames)
    
    return infoObj

