
import sys

import vhistfiles
import plugintools

from pyvhist.vhistfile import VhistFile
from pyvhist.vhistsection import VhistSection

def shallowCopy(alist):
    return alist[:]


def printFiles(files):
    print [f.filename for f in files]


# basic checks for plugin results
def checkList(hookName, pluginName, resultList):
    if not isinstance(resultList, list):
        raise plugintools.PluginError(("The %s() hook of plugin "
                "'%s' returned %s (expected a list)") %
                (hookName, pluginName, resultList))


def checkFileList(hookName, pluginName, resultList):
    if not isinstance(resultList, list):
        raise plugintools.PluginError(("The %s() hook of plugin "
                "'%s' returned %s (expected a list of VhistFile objects)") %
                (hookName, pluginName))
    
    if not all(isinstance(x, VhistFile) for x in resultList):
        raise plugintools.PluginError(("The %s() hook of plugin "
                "'%s' did not return a list of VhistFile objects") %
                (hookName, pluginName))


def checkSubset(hookName, pluginName, inputList, resultList):
    resultSet = set(resultList)
    if not resultSet.issubset(inputList):
        raise plugintools.PluginError(("The %s() hook of plugin "
                "'%s' did not return a subset of the input list")
                % (hookName, pluginName))


def checkVhistSection(hookName, pluginName, result):
    if not isinstance(result, VhistSection):
        raise plugintools.PluginError(("The %s() hook of plugin "
                "'%s' did not return a VhistFile object") %
                (hookName, pluginName))


# combined checks for plugin hooks
def checkFilterFiles(plugin, inputList, resultList):
    pluginName = plugin.__class__.getName()
    checkList("filterFiles", pluginName, resultList)
    checkSubset("filterFiles", pluginName, inputList, resultList)


def checkAmendFiles(plugin, inputList, resultList):
    pluginName = plugin.__class__.getName()
    checkList("amendFiles", pluginName, resultList)
    checkSubset("amendFiles", pluginName, inputList, resultList)


def checkConvertFiles(plugin, resultList):
    pluginName = plugin.__class__.getName()
    checkList("convertFiles", pluginName, resultList)
    checkFileList("convertFiles", pluginName, resultList)


def checkHandleSection(plugin, section):
    pluginName = plugin.__class__.getName()
    checkVhistSection("handleSection", pluginName, section)


def runPluginHook(method, *args):
    """ executes a hook, catches errors and generates sensible error
        messages """
    try:
        return method(*args)
    except Exception as e:
        pluginName = method.__self__.__class__.getName()
        hookName = method.__name__
        err = plugintools.PluginError(("hook %s() of plugin '%s' threw an "
                "error: %s") % (hookName, pluginName, str(e)))
        raise err, None, sys.exc_info()[2]

# plugin hooks
def filterFiles(files, plugins):
    keptFiles = set(files)
    for plugin in plugins:
        files2 = shallowCopy(files)
        filteredFiles = runPluginHook(plugin.filterFiles, files2)
        
        checkFilterFiles(plugin, files, filteredFiles)
        keptFiles.intersection_update(set(filteredFiles))
    keptFiles = list(keptFiles)
    
    removedFiles = set(files) - set(keptFiles)
    return keptFiles, list(removedFiles)


def amendFiles(removedFiles, plugins):
    amendedFiles = []
    for plugin in plugins:
        removedFiles2 = shallowCopy(removedFiles)
        files = runPluginHook(plugin.amendFiles, removedFiles2)
        
        checkAmendFiles(plugin, removedFiles, files)
        amendedFiles.extend(files)
    
    return list(set(amendedFiles))


def convertFiles(files, plugins):
    convertedFiles = shallowCopy(files)
    for plugin in plugins:
        convertedFiles = runPluginHook(plugin.convertFiles, convertedFiles)
        checkConvertFiles(plugin, convertedFiles)
    
    return convertedFiles


def handleSection(section, plugins):
    for plugin in plugins:
        section = runPluginHook(plugin.updateSection, section)
        checkHandleSection(plugin, section)
    
    return section


def handleEnvironment(plugins):
    for plugin in plugins:
        runPluginHook(plugin.initEnvironment)


def handleFiles(files, plugins):
    # filter and amend files
    keptFiles, removedFiles = filterFiles(files, plugins)
    amendedFiles = amendFiles(removedFiles, plugins)
    keptFiles.extend(amendedFiles)
    
    # create vhistfile objects
    vfiles = vhistfiles.createVFiles(keptFiles)
    
    # convert files and section
    convertedFiles = convertFiles(vfiles, plugins)
    vfiles, rootfile = vhistfiles.addVhistFiles(convertedFiles)
    
    return vfiles, rootfile

