
import re
import array
import binascii

import utils
from syscall import SysCall

class ParseError(RuntimeError):
    pass

# e.g. 7951 open(.....
LINE_START_REGEX = r"(?P<time>[0-9]+)\s+(?P<syscall>[a-zA-Z0-9]+)\((?P<rest>.*)"

# e.g "\x66\x72", "\x66\x72\x61\x6e\x6b\x66\x75\x72"...
STRING_REGEX = r'(\s|,)*"(?P<str>[^"]*")(\.\.\.)?(?P<rest>.*)'

# e.g. O_RDONLY|O_CLOEXEC
FLAGS_REGEX = r'(\s|,)*(?P<flags>[A-Z_|]+)(?P<rest>.*)'

# e.g. 3, -4, 123
INT_REGEX = r'(\s|,)*(?P<int>-?[0-9]+)(?P<rest>.*)'

# e.g. 3, -4, 123
OCT_REGEX = r'(\s|,)*(?P<oct>0[0-8]+)(?P<rest>.*)'

# e.g. [............]
LIST_SIMPLE_REGEX = r'(\s|,)*(?P<list>\[[^]]+\])(?P<rest>.*)'

# e.g. ) = -1 EFOOERR (foo error occured) or ) = 4
RESULT_REGEX = r'.*\)\s*=\s*(?P<result>[-0-9]+)(?P<rest>.*)'

def decodeString(string):
    # TODO: decode string using the correct encoding
    # convert \xAB\x12 to AB12
    a = array.array("c", string)
    del a[0::4]
    del a[0::3]
    newString = binascii.unhexlify(a.tostring())
    return newString


def parseRawString(restString):
    found = re.match(STRING_REGEX, restString)
    if found is not None:
        string = found.group("str")
        string = decodeString(string)
        rest = found.group("rest")
        return string, rest
    else:
        raise ParseError("Could not read string from '%s'" % restString)


def parseString(restString):
    # TODO: decode string
    return parseRawString(restString)


def parseListSimple(restString):
    found = re.match(LIST_SIMPLE_REGEX, restString)
    if found is not None:
        alist = found.group("list")
        rest = found.group("rest")
        return alist, rest
    else:
        raise ParseError("Could not read list from '%s'" % restString)

def parseFlags(restString):
    found = re.match(FLAGS_REGEX, restString)
    if found is not None:
        flags = found.group("flags").split("|")
        rest = found.group("rest")
        return flags, rest
    else:
        raise ParseError("Could not read flags from '%s'" % restString)


def parseInt(restString):
    found = re.match(INT_REGEX, restString)
    if found is not None:
        integer = int(found.group("int"))
        rest = found.group("rest")
        return integer, rest
    else:
        raise ParseError("Could not read int from '%s'" % restString)

def parseOct(restString):
    found = re.match(OCT_REGEX, restString)
    if found is not None:
        integer = int(found.group("oct"), 8)
        rest = found.group("rest")
        return integer, rest
    else:
        raise ParseError("Could not read oct value from '%s'" % restString)


def parseResult(restString):
    found = re.match(RESULT_REGEX, restString)
    if found is not None:
        result = int(found.group("result"))
        return result
    elif "<unfinished ...>" in restString:
        return None
    else:
        raise ParseError("Could not read result from '%s'" % restString)

# SYSCALL PARSERS

def parseOpen(restString):
    filename, restString = parseString(restString)
    flags, restString = parseFlags(restString)
    retval = parseResult(restString)
    return [filename, flags], retval


def parseCreat(restString):
    filename, restString = parseString(restString)
    permissions, restString = parseOct(restString)
    retval = parseResult(restString)
    return [filename, permissions], retval


def parseChdir(restString):
    directory, restString = parseString(restString)
    retval = parseResult(restString)
    return [directory], retval


def parseRename(restString):
    source, restString = parseString(restString)
    destination, restString = parseString(restString)
    retval = parseResult(restString)
    return [source, destination], retval


def parseReadWrite(restString):
    fd, restString = parseInt(restString)
    data, restString = parseRawString(restString)
    bufsize, restString = parseInt(restString)
    retval = parseResult(restString)
    return [fd, data, bufsize], retval


def parseExecve(restString):
    prog, restString = parseString(restString)
    args, restString = parseListSimple(restString)
    environ, restString = parseListSimple(restString)
    retval = parseResult(restString)
    
    return [prog, args, environ], retval


# GENERAL PARSERS

def parseLineRest(syscall, restString):
    """ General args + return value parser """
    parsers = dict(open=parseOpen, creat=parseCreat, chdir=parseChdir, 
            rename=parseRename, read=parseReadWrite, write=parseReadWrite,
            execve=parseExecve)
    
    if syscall in parsers.keys():
        return parsers[syscall](restString)
    else:
        raise ParseError("Unknown system call '%s'" % syscall)


def parseLineStart(line):
    found = re.match(LINE_START_REGEX, line)
    if found:
        timestamp = found.group("time")
        name = found.group("syscall")
        rest = found.group("rest")
        return timestamp, name, rest
    else:
        raise ParseError("Could not parse the beginning of line '%s'" % line)


def parseLine(line):
    try:
        timestamp, name, rest = parseLineStart(line)
        args, retval = parseLineRest(name, rest)
        call = SysCall(timestamp, name, args, retval)
        return call
    except ParseError as e:
        return None


def parseStrace(lines):
    # parse all lines
    syscalls = [parseLine(l) for l in lines]
    # remove Nones (unparsable lines)
    syscalls = [s for s in syscalls if s is not None]
    return syscalls


def parseStraceFile(filename):
    content = utils.readFile(filename)
    lines = content.splitlines()
    
    return parseStrace(lines)

