
import sys
import os
import glob
import textwrap
import utils

from plugins.baseplugin import BasePlugin

from pyvhist import vhistfile, userattrs

class PluginError(Exception):
    pass


def splitPluginList(plugins):
    if plugins != None:
        pluginLines = plugins.splitlines()
        allPlugins = []
        for line in pluginLines:
            allPlugins.extend(line.split(","))
            
        return [p.strip() for p in allPlugins if p.strip() != ""]
    else:
        return None


def getDefaultPlugins():
    appDir = utils.getAppDir()
    pluginCfg = os.path.join(appDir, "config/defaultplugins.cfg")
    if os.path.isfile(pluginCfg):
        with open(pluginCfg, "rb") as f:
            contents = f.read()
            return splitPluginList(contents)
    else:
        # no config -> use hard coded plugin list
        return ["unix", "embed", "env"]


def isAPluginClass(anObject):
    return isinstance(anObject, type) and issubclass(anObject, BasePlugin)


def importPlugin(pluginName, strict=False):
    # try to load plugin
    try:
        plugin = __import__("plugins." + pluginName, fromlist=pluginName)
        return plugin
    except Exception as e:
        raise PluginError("Failed to load plugin %s: %s" % (pluginName,
            str(e))), None, sys.exc_info()[2]


def isAPlugin(pluginName, plugin, strict=False):
    # check if the module is a plugin
    if plugin.__dict__.get("VHISTIFY_NOT_A_PLUGIN", False):
        if strict:
            raise PluginError(("Failed to load plugin %s: is not a plugin "
                    "(VHISTIFY_NOT_A_PLUGIN set in module)") % pluginName)
        else:
            return False
    return True


def getPluginClass(pluginName, plugin, strict=False):
    # check if the module contains exactly one plugin class
    pluginClasses = [entry for entry in plugin.__dict__.values() if
            isAPluginClass(entry)]
    if len(pluginClasses) == 0:
        if strict:
            raise PluginError("Failed to load plugin %s: module does not "
                    "contain a plugin class" % pluginName)
        else:
            return None
    elif len(pluginClasses) == 1:
        return pluginClasses[0]
    else:
        raise PluginError("module %s contains several plugin classes"
                % plugin)


def loadPlugin(pluginName, strict=False):
    plugin = importPlugin(pluginName, strict)
    if not isAPlugin(pluginName, plugin, strict):
        return None
    return getPluginClass(pluginName, plugin, strict)
    

def loadPlugins(pluginNames, strict=False):
    plugins = []
    for pluginName in pluginNames:
        plugin = loadPlugin(pluginName, strict)
        if plugin is not None:
            plugins.append(plugin)
    return plugins


def genDocString(docstring, width):
    indentSize = 4
    # remove widespace from docstring
    lines = docstring.splitlines()
    lines = [l.strip() for l in lines]
    
    # remove leading and trailing empty lines
    while len(lines) > 0 and lines[0] == "":
        del lines[0]
    
    while len(lines) > 0 and lines[-1] == "":
        del lines[-1]
    
    # split text into whitespace separated blocks
    blocks = []
    blockStart = 0
    for i, line in enumerate(lines):
        if line == "":
            blockEnd = i
            blocks.append(lines[blockStart:blockEnd])
            blockStart = i+1
    blocks.append(lines[blockStart:])
    
    # join each block into one string
    docstrings = ["\n".join(block) for block in blocks]
    
    # format block docstrings
    newBlocks = [textwrap.wrap(docstring, width - indentSize) for docstring
            in docstrings]
    
    # merge all blocks into one list of lines
    lines = []
    for block in newBlocks:
        lines.extend(line for line in block)
        lines.append("")
    lines = [" " * indentSize + l.strip() for l in lines]
    return lines


def getPluginHelp(pluginName, columns):
    try:
        pluginClass = loadPlugin(pluginName)
        
        if pluginClass == None:
            docString = "Could not load plugin %s" % pluginName
        else:
            docString = pluginClass.getDocumentation()
    except ImportError as e:
        docString = "Plugin %s does not exist" % pluginName
    except Exception as e:
        docString = "Error while loading plugin %s:\n %s" % (pluginName, e)
    return genDocString(docString, columns)


def getAllPluginClasses():
    pattern = os.path.join(utils.getPluginDir(), "*.py")
    files = glob.glob(pattern)
    
    pluginClasses = []
    for f in files:
        basename = os.path.basename(f)
        basename, ext = os.path.splitext(basename)
        
        try:
            plugin = loadPlugin(basename)
            if plugin is not None:
                pluginClasses.append(plugin)
        except Exception as e:
            raise
    
    return pluginClasses


def createPluginList(width, defaultPluginNames):
    plugins = getAllPluginClasses()
    lines = []
    for plugin in plugins:
        try:
            pluginName = plugin.getName()
            if pluginName in defaultPluginNames:
                lines.append("* %s [default]" % pluginName)
            else:
                lines.append("* %s" % pluginName)
            
            if plugin.__doc__ is not None:
                lines.extend(genDocString(plugin.__doc__, width))
        except Exception as e:
            raise
    return lines


def createPluginInstances(pluginClasses, infoObj, tmpdir):
    plugins = []
    for pluginClass in pluginClasses:
        plugin = pluginClass()
        plugin._setup(infoObj, tmpdir, vhistfile, userattrs)
        plugins.append(plugin)
    return plugins


def getPlugins(infoObj, tmpdir):
    # get plugins
    defaultPluginNames = infoObj.defaultPluginNames
    otherPluginNames = infoObj.pluginNames
    allPluginNames = defaultPluginNames + otherPluginNames
    pluginClasses = loadPlugins(allPluginNames, strict=True)
    plugins = createPluginInstances(pluginClasses, infoObj, tmpdir)
    return plugins

