"""
This module contains a logger, which monitors stdout and stderr and stores
the information written to it into a temporary file. The Logger uses should
be used with python's ``with`` statement. You can write::
    
    from pyvhist.logger import Logger
    from pyvhist.tempdir import TempDir
    
    logfile = "output.log"
    with Logger(logfile, tempDir) as log:
        doStuff()
    
    addFileToVhistSection(log.filename)

You can use the property ``filename`` after your code left the ``with``-block
to access the filename of the temporary file, which contains the log output.

"""

import sys

class Logger(object):
    """ A class, which monitors stdout and stderr """
    def __init__(self, filename, tempDir):
        self.filename = tempDir.createTempFile(filename)
        self.logfile = None
        self.oldStdout = None
        self.oldStderr = None
    
    def write(self, data):
        self.logfile.write(data)
        self.oldStdout.write(data)
    
    def flush(self):
        self.logfile.flush()
        self.oldStdout.flush()
    
    def __enter__(self):
        self.oldStdout = sys.stdout
        self.oldStderr = sys.stderr
        self.logfile = file(self.filename, "w")
        sys.stdout = self
        sys.stderr = self
        return self
    
    def __exit__(self, type, value, traceback):
        sys.stdout = self.oldStdout
        sys.stderr = self.oldStderr
        self.logfile.close()

