"""
Each VHIST workflow step and workflow file allow you to add arbitrary
user-defined attributes in the form of key-value pairs. The classes in
this module allow you to easily specify them and reuse their definitions
within your code. This way, you do not have to specify each attribute
twice, such as::

    from pyvhist.vhistfile import InFile
    
    # create constants used in your code
    IMAGE_WIDTH = 200
    IMAGE_HEIGHT = 150
    IMAGE_DEPTH = 32
    IMAGE_SEGMENTS = 5
    
    # add each constant to the vhist workflow file
    vfile = InFile("image.png")
    vfile.userAttrs["imageWidth"] = IMAGE_WIDTH
    vfile.userAttrs["imageHeight"] = IMAGE_HEIGHT
    vfile.userAttrs["imageDepth"] = IMAGE_DEPTH
    vfile.userAttrs["imageSegments"] = IMAGE_SEGMENTS
    
    # use the constants in your code
    for i in range(IMAGE_WIDTH):
        for j in range(IMAGE_HEIGHT):
            image[i,j] += 10

Instead, you can create an object of type :class:`UserAttributes`,
add all parameters to this class and then include the user attributes
with one call into a vhist workflow step or file::

    from pyvhist.userattrs import UserAttributes
    from pyvhist.vhistfile import InFile
    
    # create constants used in your code
    P = UserAttributes()
    P.imageWidth = 200
    P.imageHeight = 150
    P.imageDepth = 32
    P.imageSegments = 5
    
    # add all constants at once to the workflow file
    vfile = InFile("image.png")
    P.addToUserAttrs(vfile)
    
    # use P in your code
    for i in range(P.imageWidth):
        for j in range(P.imageHeight):
            image[i,j] += 10

Each user-defined attribute can have a unit. For example, if you deal with
lengths, you want to document if the lenghts are in mm, inches, pixels or
light-years. You can document the unit of a user-defined attribute with
the help of the UserValue class::

    from pyvhist.userattrs import UserAttributes, UserValue
    P = UserAttributes()
    P.imageWidth = UserValue(200, unit="pixels")
    P.imageHeight = UserValue(150, unit="pixels")
    P.imageDepth = UserValue(32, unit="bits")

While units are important when documenting attributes, they are not
used in code very often. Therefore, :class:`UserAttributes` automatically 
detects if you assign a :class:`UserValue` object to it. When you access
this property, only the value stored in the :class:`UserValue` object is
returned::

    P = UserAttributes()
    P.imageWidth = UserValue(200, unit="pixels")
    
    print P.imageWidth # returns only the value 200

User Value also provides a keyword option ``formatStr`` in the constructor.
This option can be used to define how number-like objects are displayed
within the VHIST file::

    P = UserAttributes()
    P.radius = UserValue(10, unit="mm")
    P.circumference = UserValue(2 * math.pi * P.radius, unit="mm",
            formatStr="%.3f")
"""

from external.ordereddict.ordereddict import OrderedDict

# prettyX based on an idea from
# http://stackoverflow.com/questions/1447287/format-floats-with-standard-json-module

def genPrettyFloat(value, formatStr):
    class PrettyFloat(float):
        def __repr__(self):
            if formatStr is not None:
                return formatStr % self
            else:
                return repr(float(self))
    return PrettyFloat(value)


def genPrettyInt(value, formatStr):
    class PrettyInt(int):
        def __repr__(self):
            if formatStr is not None:
                return formatStr % self
            else:
                return repr(int(self))
    return PrettyInt(value)


def genPrettyLong(value, formatStr):
    class PrettyLong(long):
        def __repr__(self):
            if formatStr is not None:
                return formatStr % self
            else:
                return repr(long(self))
    return PrettyInt(value)


class UserValue(object):
    """ A class, which can store a value, an associated unit and a
    format string."""
    def __init__(self, value, unit=None, formatStr=None, verbatim=False):
        """ Creates a ``UserValue`` object.

        ``unit`` is a string, which specifies the unit associated with the
        value.
        
        ``formatStr`` is a string, which is used to format numeric values.
        """
        self.value = value
        self.unit = unit
        self.formatStr = formatStr
        self.verbatim = verbatim


class UserAttributes(object):
    """ A class, which stores key-value pairs.
    """
    def __init__(self):
        """ Create a ``UserAttributes`` object. """
        self.__itemOrder = []
        self.__values = {}
    
    def __setattr__(self, key, value):
        if "_UserAttributes__itemOrder" in dir(self) and \
                key not in self.__itemOrder \
                and key != "_UserAttributes__values":
            self.__itemOrder.append(key)
        if "_UserAttributes__values" in dir(self):
            self.__values[key] = value
        
        if isinstance(value, UserValue):
            value = value.value
        
        object.__setattr__(self, key, value)
    
    def addToUserAttrs(self, secOrFile, prefix=""):
        """ Adds the user-defined attributes stored in the
        :class:`UserAttributes` object to the %%vhistsection.VhistSection
        or %%vhistfile.VhistFile object given by ``secOrFile``.

        ``prefix`` is an optional parameter. If ``prefix`` is set to a
        non-empty string, all parameters will be prefixed with the
        prefix followed by a '.'.
        """
        for key in self.__itemOrder:
            value = self.__values[key]
            self.addOneItemToUserAttrs(secOrFile, key, value, prefix)
    
    def addOneItemToUserAttrs(self, secOrFile, key, value, prefix=None):
        if prefix != None:
            newPrefix = "%s.%s" % (prefix, key)
        else:
            newPrefix = key
        
        if isinstance(value, UserAttributes):
            value.addToUserAttrs(secOrFile, newPrefix)
        else:
            secOrFile.userAttrs[newPrefix] = value


