
from __future__ import division
import pdflayouter, pdfutils, workflow

# millimeter to points ratio
MM2PT = 72 / 25.4
CHARS_PER_COL = 79
LINES_PER_PAGE = 63

# ll_x, ll_y, ur_x, ur_y of media box (in pt, paper size => A4)
MEDIABOX_DIMENSIONS = [0, 0, 595, 842]
MEDIABOX_RATIO = MEDIABOX_DIMENSIONS[2] / MEDIABOX_DIMENSIONS[3]

# position of the left edge of the two columns on figure pages (in pt)
COL1LEFT = 60
COL2LEFT = 313
COLLEFT = (COL1LEFT, COL2LEFT)

# position of the right edge of the two columns on figure pages (in pt)
COL1WIDTH = COL2WIDTH = 222
COL1RIGHT = COL1LEFT + COL1WIDTH
COL2RIGHT = COL2LEFT + COL2WIDTH
COLRIGHT = (COL1RIGHT, COL2RIGHT)

# top and bottom a figure page column (in pt)
COLTOP = 760
COLBOTTOM = 64
COLHEIGHT = COLTOP - COLBOTTOM

# width of border around images (in pt)
IMAGEBORDER = 5

# image left + right of an image (in pt, without border, column 1)
IMAGE1LEFT  = COL1LEFT + IMAGEBORDER
IMAGE1WIDTH = COL1WIDTH - 2 * IMAGEBORDER
IMAGE1RIGHT = IMAGE1LEFT + IMAGE1WIDTH

# image left + right of an image (in pt, without border, column 2)
IMAGE2LEFT  = COL2LEFT + IMAGEBORDER
IMAGE2WIDTH = COL2WIDTH - 2 * IMAGEBORDER
IMAGE2RIGHT  = IMAGE2LEFT + IMAGE2WIDTH

# distance between image and caption
IMAGECAPTIONDIST = 10

# distance between figures
FIGUREDIST = 20

def sortHori(a, b):
    if a <= b:
        return a, b
    else:
        return b, a


def sortVert(a, b):
    if a >= b:
        return a, b
    else:
        return b, a


class LayoutItem(object):
    def __init__(self, x, y):
        self.x = x
        self.y = y
    
    def getExtent(self, offx = 0, offy = 0):
        return None
    
    def toPdfStream(self, offx = 0, offy = 0):
        return None
    
    def getSize(self):
        extent = self.getExtent()
        width = extent[1] - extent[0]
        height = extent[2] - extent[3]
        
        return width, height


class LayoutImage(LayoutItem):
    def __init__(self, x, y, width, height, pdfname):
        LayoutItem.__init__(self, x, y)
        self.width = width
        self.height = height
        self.pdfname = pdfname
    
    def getExtent(self, offx = 0, offy = 0):
        x1 = self.x + offx
        x2 = x1 + self.width
        y1 = self.y + offy
        y2 = y1 - self.height
        x1, x2 = sortHori(x1, x2)
        y1, y2 = sortVert(y1, y2)
        
        return (x1 - IMAGEBORDER, x2 + IMAGEBORDER, y1 + IMAGEBORDER,
                y2 - IMAGEBORDER)
    
    def toPdfStream(self, offx = 0, offy = 0):
        rawStream = ("%% begin rectangle\n"
        "0.8 0.8 0.8 rg\n"
        "%f %f %f %f re\n"
        "f\n"
        "%% image begin\n"
        "q\n"
        "1 0 0 1 %f %f cm\n"
        "%f 0 0 %f 0 0 cm\n"
        "%s Do\n"
        "Q\n"
        "%% image end\n")
        
        extent = self.getExtent(offx, offy)
        return rawStream % (extent[0], extent[3], self.width + 2 * IMAGEBORDER,
                self.height + 2 * IMAGEBORDER,
                extent[0] + IMAGEBORDER, extent[3] + IMAGEBORDER, self.width,
                self.height, self.pdfname)


class LayoutText(LayoutItem):
    def __init__(self, x, y, formatedText, font, fontSize, markers):
        LayoutItem.__init__(self, x, y)
        self.text = formatedText
        self.font = font
        self.fontSize = fontSize
        self.markers = markers
    
    def getNumberOfNewlines(self):
        newlines = [t for t in self.text if t["ID"] == pdflayouter.ID_NEWLINE]
        return len(newlines)
    
    def getLines(self):
        lines = []
        curline = []
        for token in self.text:
            if token["ID"] == pdflayouter.ID_NEWLINE:
                lines.append("".join(curline))
                curline = []
            else:
                curline.append(token["TEXT"])
        if len(curline) > 0:
            lines.append("".join(curline))
        
        return lines
    
    def getLongestLineLength(self):
        lines = self.getLines()
        return max(len(line) for line in lines)
    
    def getExtent(self, offx = 0, offy = 0):
        # TODO add sizes for different default fonts
        x1 = self.x + offx
        y1 = self.y + offy
        x2 = x1 + self.getLongestLineLength() * 7
        y2 = y1 - (self.getNumberOfNewlines() + 1) * (self.fontSize + 1)
        return (x1, x2, y1, y2)
    
    def toPdfStream(self, offx = 0, offy = 0):
        x = self.x + offx
        y = self.y + offy - self.fontSize + 2
        textStream = self.text
        defaultFont = self.font
        size = self.fontSize
        
        black = "0 0 0 rg"
        blue = "0.30 0.30 0.86 rg"
        fonts = {None: defaultFont, "*": "/F3", "_": "/F4", "=": defaultFont}
        colors = {None: black, "*": black, "_": black, "=": blue}
        
        streamList = []
        streamList.append("% text begin")
        streamList.append("BT")
        streamList.append("%d %d Td" % (x, y))
        streamList.append("%d TL" % (size + 1))
        for token in textStream:
            if token["ID"] == pdflayouter.ID_TEXT:
                font = fonts[token["FORMAT"]]
                color = colors[token["FORMAT"]]
                streamList.append(color)
                streamList.append("%s %d Tf" % (font, size))
                pdfString = pdfutils.makePDFString(token["TEXT"])
                streamList.append("%s Tj" % pdfString)
            elif token["ID"] == pdflayouter.ID_NEWLINE:
                y += size
                streamList.append("T*")
            else:
                raise Exception, "Unreachable passage reached"
        streamList.append("ET")
        streamList.append("% text end")
        streamList.append("")
        stream = "\n".join(streamList)
        return stream

"""
class LayoutLine(LayoutItem):
    def __init__(self, x1, y1, x2, y2, lineWidth):
        LayoutItem.__init__(self, x, y)
        self.text = formatedText
        self.font = font
        self.fontSize = fontSize
        self.markers = markers
    
    def toPdfStream(self, offx, offy):
        x1 = self.x1 + offx
        x2 = self.
        rawLine = ("%% line begin\n"
               "%f %f m\n"
               "%f %f l\n"
               "%f w\n"
               "S\n"
               "%% line end\n")
        return rawLine % (x1_pt, y1_pt, x2_pt, y2_pt, width_pt)
"""

class LayoutGroup(LayoutItem):
    def __init__(self, x, y, items):
        self.x = x
        self.y = y
        self.items = items
    
    def getExtent(self, offx = 0, offy = 0):
        inf = float("inf")
        extent = [inf, -inf, -inf, inf]

        for item in self.items:
            e = item.getExtent(self.x + offx, self.y + offy)
            extent[0] = min(extent[0], e[0])
            extent[1] = max(extent[1], e[1])
            extent[2] = max(extent[2], e[2])
            extent[3] = min(extent[3], e[3])

        return tuple(extent)

    def toPdfStream(self, offx = 0, offy = 0):
        streams = []
        
        for item in self.items:
            streams.append(item.toPdfStream(self.x + offx, self.y + offy))
        
        return "\n".join(streams)


def addStandardLayout(contentObj, titleStream, bodyStream, footerStream):
    """
    Takes text for title, body and footer part of the 
    page and adds it to the PDF content object; text is supposed
    to returns the PDF stream for setting the text
    
    """
    body = []
    for item in bodyStream:
        body.append(item.toPdfStream())
    bodyStream = "\n".join(body)

    contentObj.appendRawStream("% VHIST standard layout\n")
    contentObj.addLine(60, 785, 535, 785, 2.0)
    contentObj.addLine(60, 782, 535, 782, 1.0)
    contentObj.addLine(60, 57, 535, 57, 1.0)
    contentObj.addText(60, 790, titleStream, "/F1", 10)
    contentObj.addText(60, 46, footerStream, "/F1", 10)
    contentObj.appendRawStream(bodyStream)
    #contentObj.addText(60, 760, bodyStream, "/F2", 10)


def layoutSummary(summaryText, supportMarkers = True):
    layouter = pdflayouter.PDFLayouter(CHARS_PER_COL, LINES_PER_PAGE,
            supportMarkers)
    pages = layouter.layoutText(summaryText)
    allMarkers = [layouter.getMarkers(i) for i in xrange(0, len(pages))]
    
    layoutPages = []
    for p, m in zip(pages, allMarkers):
        ypos = 760 + 10 - 2 # -fontsize + 2 offset in LayoutText. HACK!!!
        p = [LayoutText(60, ypos, p, "/F2", 10, m)]
        layoutPages.append(p)
    
    return layoutPages, allMarkers


def layoutFigure(image):
    """ Layouts an image (including background and caption) and returns a
    LayoutGroup object containing the components.
    """
    maxw = IMAGE1WIDTH
    
    # get image size (in pixels)
    w = image.width
    h = image.height
    
    # scale image to fit width of column (in points) 
    scale = maxw / w
    w = w * scale
    h = h * scale
    
    limg = LayoutImage(IMAGEBORDER, -IMAGEBORDER, w, h, image.pdfName)
    y = limg.getExtent()[3] - IMAGECAPTIONDIST
    
    escape = workflow.generateOneSummaryString
    
    caption = "*%s*: %s" % (escape(image.displayname), escape(image.caption))
    
    # layout image description using layout engine (cut after 10 lines)
    layouter = pdflayouter.PDFLayouter(47, 10, True)
    captionpages = layouter.layoutText(caption)
    caption = captionpages[0]
    lcaption = LayoutText(0, y, caption, "/F2", 8, None)
    
    group = LayoutGroup(0, 0, [limg, lcaption])
    return group


def layoutPreviews(images):
    """ Takes a number of images and layouts preview pages for these images
    """
    pages = []
    y = COLTOP
    heightLeft = COLHEIGHT
    column = 0 # possible values are 0, 1 in a two column layout
    
    figures = [layoutFigure(image) for image in images]
    curpage = []
    
    for figure in figures:
        w, h = figure.getSize()
        if h > heightLeft and heightLeft < COLHEIGHT:
            if column == 1:
                pages.append(curpage)
                curpage = []
            column = (column + 1) % 2
            y = COLTOP
            heightLeft = COLHEIGHT
            
        figure.x = COLLEFT[column]
        figure.y = y
        curpage.append(figure)
        y -= (h + FIGUREDIST)
        heightLeft -= (h + FIGUREDIST)
        
    if len(curpage) > 0:
        pages.append(curpage)
    return pages

