"""
Module containing a lexer specifically designed to lex (uncompressed)
pdf objects.



This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This application is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

$Id: pdflexer.py 1895 2009-11-16 18:32:45Z ahuesgen $
"""

import re
import pdfutils, lexer

# IDs used to identify different token types.
ID_BOOL = 1; ID_NULL = 2; ID_ARRAY_BEGIN = 3; ID_ARRAY_END = 4;
ID_DICT_BEGIN = 5; ID_DICT_END = 6; ID_NUMBER = 7; ID_NAME = 8;
ID_HEXSTR = 9; ID_LITSTR = 10; ID_STREAM = 11; ID_OBJ = 12; ID_REF = 13;
ID_ENDOBJ = 14


# A dictionary containing string versions of the token IDs.
IDSTRINGS = {ID_BOOL: "Bool", ID_NULL: "Null", ID_ARRAY_BEGIN: "Array begin",
             ID_ARRAY_END: "Array end", ID_DICT_BEGIN: "Dict begin",
             ID_DICT_END: "Dict end", ID_NUMBER: "Number", ID_NAME: "Name",
             ID_HEXSTR: "Hex string", ID_LITSTR: "Literal string",
             ID_STREAM: "stream", ID_OBJ: "Obj", ID_REF: "Ref",
             ID_ENDOBJ: "Endobj"}


def __genToken(match, id):
    """
    Generates a token dictionary for the given match result.
    
    @type  match: re.MatchObject
    @param match: The match result, from which the token should be constructed.
    
    @type  id: int
    @param id: The id of the token type.
    
    @rtype:  None or dict
    @return: token dictionary with fields "ID" (int; id of the token type),
             "POS" (int; startposition of token in string),
             "ENDPOS" (int; endposition of token in string),
             "TEXT" (str; text recognized as token). and
             "VALUE" (undefined type; value, which can be included into a
             PDFStructObject). If id is None, None is returned.
    
    """
    if id is None:
        return None
    dict = {}
    dict["ID"]   = id
    dict["POS"]  = match.pos
    dict["ENDPOS"]  = match.pos + len(match.group(0))
    dict["TEXT"] = match.group(0)
    dict["VALUE"] = None
    return dict

def __genBoolToken(match, id):
    dict = __genToken(match, id)
    dict["VALUE"] = pdfutils.makePDFBool(dict["TEXT"])
    return dict

def __genNumberToken(match, id):
    dict = __genToken(match, id)
    dict["VALUE"] = pdfutils.makePDFNumber(dict["TEXT"])
    return dict

def __genStringToken(match, id):
    dict = __genToken(match, id)
    dict["VALUE"] = pdfutils.normalizePDFString(dict["TEXT"])
    return dict

def __genNameToken(match, id):
    dict = __genToken(match, id)
    dict["VALUE"] = pdfutils.normalizePDFName(dict["TEXT"])
    return dict

def __genDictToken(match, id):
    dict = __genToken(match, id)
    dict["VALUE"] = pdfutils.makePDFDict()
    return dict

def __genArrToken(match, id):
    dict = __genToken(match, id)
    dict["VALUE"] = pdfutils.makePDFArray()
    return dict

def __genNullToken(match, id):
    dict = __genToken(match, id)
    dict["VALUE"] = pdfutils.makePDFNull()
    return dict

def __genRefToken(match, id):
    dict = __genToken(match, id)
    dict["OBJID"] = int(match.group("id"))
    dict["OBJGEN"] = int(match.group("gen"))
    dict["VALUE"] = pdfutils.makePDFRef(dict["OBJID"], dict["OBJGEN"])
    return dict

def __genObjToken(match, id):

    dict = __genToken(match, id)
    dict["OBJID"] = int(match.group("id"))
    dict["OBJGEN"] = int(match.group("gen"))
    return dict


def genRange(list):
    str = "["
    for i in list:
        str += "\\%03o" % i
    return str + "]"

WS_SET   = set([0,9,10,12,13,32])
DE_SET   = set([37,40,41,47,60,62,91,93,123,125])
PR_SET   = set(range(33, 256)) - set([128])
RE_SET   = set(range(0, 256)) - (WS_SET | DE_SET)
STRRE_SET = set(range(0, 256)) - set([40, 41, 92])
NAMERE_SET = RE_SET - set([35])

CSETS = {}
# whitespace characters (\r\n, \r, \n, \f, SPACE, \t and \0)
CSETS["WS"]      = "((\r\n)|" + genRange(WS_SET) + ")"
# space and tab
CSETS["SP"]      = "[\t ]"
# newline characters (\r\n, \r and \n)
CSETS["NL"]      = "((\r\n)|[\n\r])"
# pdf delimiter characters (%, /, <, >, (, ), [, ], { and })
CSETS["DE"]      = genRange(DE_SET)
# printable characters (also > 127 characters)
CSETS["PR"]      = genRange(PR_SET)
# pdf regular characters (all except whitespace and delimiter characters)
CSETS["RE"]      = genRange(RE_SET)
# all characters, that may appear in a pdf name (without escapes)
CSETS["NAMERE"]  = genRange(NAMERE_SET)
# the format of an escape sequence in a pdf name
CSETS["NAMEESC"] = "((#[0-9A-Fa-f][1-9A-Fa-f])|(#[1-9A-Fa-f][0-9A-Fa-f]))"
# the format of an escape sequence in a pdf literal string
CSETS["STRESC"]  = "((\\\\[nrbtf\\\\()])|(\\\\%(NL)s)|(\\\\[0-9]{1,3}))" % CSETS
# all characters, that may appear in a pdf literal string (UNSURE)
CSETS["STRRE"]   = genRange(STRRE_SET)


# regular expressions for all major pdf tokens
REGSTR_COMMENT     = ("%%((%(PR)s)|%(SP)s)*%(NL)s|" + \
                      "%%((%(PR)s)|%(SP)s)*$") % CSETS
REGSTR_WHITE       = "%(WS)s+" % CSETS
REGSTR_BOOL        = "(true)|(false)"
REGSTR_NULL        = "null"
REGSTR_ARRAY_BEGIN = "\\["
REGSTR_ARRAY_END   = "\\]"
REGSTR_DICT_BEGIN  = "<<"
REGSTR_DICT_END    = ">>"
REGSTR_NUMBER      = "([+-]?[0-9]+\.?[0-9]*)|([+-]?[0-9]*\.[0-9]+)"
REGSTR_NAME        = "\\/(%(NAMEESC)s|%(NAMERE)s)*" % CSETS
REGSTR_HEXSTR      = "<([0-9a-fA-F]|%(WS)s)*>" % CSETS
REGSTR_LITSTR      = "\\(((%(STRRE)s)|%(SP)s|%(STRESC)s)*\\)" % CSETS
REGSTR_STREAM      = "stream\r?\n"
REGSTR_OBJ         = "(?P<id>[0-9]+)%(WS)s+(?P<gen>[0-9]+)%(WS)s+obj" % CSETS
REGSTR_REF         = "(?P<id>[0-9]+)%(WS)s+(?P<gen>[0-9]+)%(WS)s+R" % CSETS
REGSTR_ENDOBJ      = "endobj"
REGSTR_STARTXREF   = "startxref"
REGSTR_TRAILER     = "trailer"

LEXEXPR_COMMENT     = (REGSTR_COMMENT, False, __genToken, None)
LEXEXPR_WHITE       = (REGSTR_WHITE, False, __genToken, None)
LEXEXPR_BOOL        = (REGSTR_BOOL, False, __genBoolToken, ID_BOOL)
LEXEXPR_NULL        = (REGSTR_NULL, False, __genNullToken, ID_NULL)
LEXEXPR_ARRAY_BEGIN = (REGSTR_ARRAY_BEGIN, False, __genArrToken, ID_ARRAY_BEGIN)
LEXEXPR_ARRAY_END   = (REGSTR_ARRAY_END, False, __genToken, ID_ARRAY_END)
LEXEXPR_DICT_BEGIN  = (REGSTR_DICT_BEGIN, False, __genDictToken, ID_DICT_BEGIN)
LEXEXPR_DICT_END    = (REGSTR_DICT_END, False, __genToken, ID_DICT_END)
LEXEXPR_NUMBER      = (REGSTR_NUMBER, False, __genNumberToken, ID_NUMBER)
LEXEXPR_NAME        = (REGSTR_NAME, False, __genNameToken, ID_NAME)
LEXEXPR_HEXSTR      = (REGSTR_HEXSTR, False, __genStringToken, ID_HEXSTR)
LEXEXPR_LITSTR      = (REGSTR_LITSTR, False, __genStringToken, ID_LITSTR)
LEXEXPR_STREAM      = (REGSTR_STREAM, True, __genToken, ID_STREAM)
LEXEXPR_OBJ         = (REGSTR_OBJ, False, __genObjToken, ID_OBJ)
LEXEXPR_REF         = (REGSTR_REF, False, __genRefToken, ID_REF)
LEXEXPR_ENDOBJ      = (REGSTR_ENDOBJ, True, __genToken, ID_ENDOBJ)
LEXEXPR_STARTXREF   = (REGSTR_STARTXREF, True, __genToken, None)
LEXEXPR_TRAILER     = (REGSTR_TRAILER, False, __genToken, None)


LEXEXPR_ALL = [LEXEXPR_WHITE,
               LEXEXPR_OBJ, LEXEXPR_REF, LEXEXPR_ENDOBJ, LEXEXPR_STREAM,
               LEXEXPR_STARTXREF, LEXEXPR_TRAILER,
               LEXEXPR_BOOL, LEXEXPR_NULL,
               LEXEXPR_ARRAY_BEGIN, LEXEXPR_ARRAY_END,
               LEXEXPR_DICT_BEGIN, LEXEXPR_DICT_END,
               LEXEXPR_COMMENT, LEXEXPR_NAME,
               LEXEXPR_NUMBER, LEXEXPR_HEXSTR, LEXEXPR_LITSTR]


class PDFLexer(lexer.Lexer):
    """
    A Lexer designed to lex PDF objects.
    
    """
    def __init__(self):
        """
        Creates A PDFLexer.
        
        """
        lexer.Lexer.__init__(self)
        for regex in LEXEXPR_ALL:
            cregex = re.compile(regex[0])
            lexExpr = lexer.LexExpr(cregex, regex[1], regex[2], regex[3])
            self.addLexExpr(lexExpr) 
