"""
Module containing functions for populating a pdf file with objects and
layouting pages.



This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This application is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

$Id: pdfmain.py 2670 2013-04-15 10:34:13Z ahuesgen $
"""

import os, shutil

import logger, newvfile, addsection
from utilities import assertion


def samefile(src, dest):
    try:
        return os.path.samefile(src, dest)
    except:
        # Windows does not support os.path.samefile
        return os.path.abspath(src) == os.path.abspath(dest)


def copyfile(src, dest, overwrite = False):
    if not samefile(src, dest):
        if not os.path.exists(dest) or overwrite:
            try:
                shutil.copyfile(src, dest)
            except IOError, e:
                raise
        else:
            raise IOError, ("error while copying file '%s' to '%s'. '%s' "
                    "already exists") % (src, dest, dest)


def initPrimaryOutputFile(wf):
    EXISTS_ERROR = ("output vhistfile '%s' does already exist. "
            "If you want to append to a VHIST file, you also need to "
            "specify it with the '-I' option.")
    assertion(len(wf.genVhistFiles) != 0, "No output vhistfiles specified")
    primaryOutputFile = wf.genVhistFiles[0]
    
    # root file exists ... copy and use it
    if os.path.isfile(wf.vhistRootFile):
        copyfile(wf.vhistRootFile, primaryOutputFile)
    # either no root file and optional root file...
    elif wf.rootIsOptional or wf.vhistRootFile == u"":
        if os.path.exists(primaryOutputFile):
            raise IOError, EXISTS_ERROR % primaryOutputFile
    # root file not found but needed ... error
    else:
        raise IOError, "vhist root file '%s' does not exist" % wf.vhistRootFile
    return primaryOutputFile


def createSecondaryOutputFiles(primaryOutputFile, secondaryOutputFiles,
        rootFile):
    # duplicate vhist file to all output files
    for vOutfile in secondaryOutputFiles:
        logger.log(2, "copying file '%s' to '%s'", primaryOutputFile,
                vOutfile)
        # vhist root file may be overwritten
        overwrite = samefile(rootFile, vOutfile)
        copyfile(primaryOutputFile, vOutfile, overwrite)


def addWorkflowStep(wf, readme, titleContents):
    """
    Takes information from workflow class wf, gathers information
    about argument files, does basic text formatting, adds appropriate
    PDF objects and writes the PDF file.
    
    @type  wf: workflow.Workflow
    @param wf: The workflow which should be added to the section.
    
    @type  readme: str
    @param readme: The readme file, which should be added in the vhistfile.
    
    @type  titlecontent: unicode
    @param titlecontent: The content of the titlepage of the vhistfile.
    
    """
    primaryOutputFile = initPrimaryOutputFile(wf)
    
    # a new vhistfile? init with standard objects and add titlepage section
    if not os.path.exists(primaryOutputFile):
        newvfile.createNewVhistFile(primaryOutputFile, wf, readme,
                titleContents)
    
    # append to an existing file
    if os.path.isfile(primaryOutputFile):
        addsection.appendSection(primaryOutputFile, wf)
        
        createSecondaryOutputFiles(primaryOutputFile, wf.genVhistFiles[1:],
                wf.vhistRootFile)

