"""
Module containing classes for representing PDF objects.



This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This application is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

$Id: pdfobjs.py 1071 2008-05-25 18:58:08Z ahuesgen $
"""

import re
import utilities, pdfutils, pdftypes


class PDFObjectError(RuntimeError):
    """
    An error which occured in a pdf object.
    
    """
    pass


class PDFPathError(RuntimeError):
    """
    An error which occures during the creation of a PDFPath object.
    
    """
    pass


class PDFObject(pdftypes.PDFRef):
    """
    The baseclass for all objects in a pdf file. All pdf objects derived from
    this class can be converted into a string with the toStr() method. Since
    PDFObject is derived from PDFRef, every pdf object can also be used as a
    reference to itself.
    
    """
    def __init__(self, id, gen = 0):
        """
        Creates a new PDFObject with the given id and generation.
        
        @type  id: int
        @param id: The id (object number) of the pdf object. Accessible through
                   the member id.
        
        @type  gen: int
        @param gen: The generation number of the pdf object. Accessible through
                    the member gen.
        
        """
        pdftypes.PDFRef.__init__(self, id, gen)
    
    def __getHeader(self):
        """
        Returns the header of the pdf object.
        
        @rtype:  str
        @return: The objects header (x y obj).
        
        """
        return "%d %d obj\n" % (self.id, self.gen)
    
    def __getFooter(self):
        """
        Returns the endobj terminator of the pdf object.
        
        @rtype:  str
        @return: The objects footer (endobj).
        
        """
        return "endobj\n\n"
    
    def getObjectString(self, objectBody):
        """
        Returns a string consisting of the header, body and footer of the
        object.
        
        @type  objectBody: str
        @param objectBody: A string containing the body of the pdf object.
        
        @rtype:  str
        @return: A string containing a complete pdf object.
        
        """
        return self.__getHeader() + objectBody + self.__getFooter()
    
    def toStr(self):
        """
        Returns a string containing the pdf object.
        This string can be included into the pdf file.
        
        @rtype:  str
        @return: A string containing the serialized form of the pdf object.
        
        """
        return self.getObjectString("")
    
    def getXRefEntry(self):
        """
        Returns an XRefEntry object for the pdf object.
        
        @rtype:  xref.XRefEntry
        @return: An XRefEntry object for this pdf object. The position of the
                 object is set to 0.
        
        """
        return xref.XRefEntry(0, self.id, self.gen)


class PDFSimpleObject(PDFObject):
    """
    A PDFObject with an id, a generation and a simple text body. This
    PDFObject is a generalized object which can be used for many different
    pdf object types.
    
    """
    def __init__(self, id, gen, body):
        """
        Create a new PDFSimpleObject.
        
        @type  id: int
        @param id: The id of the pdf object.
        
        @type  gen: int
        @param gen: The generation number of the pdf object.
        
        @type  body: str
        @param body: A string with the body of the pdf object.
        
        """
        PDFObject.__init__(self, id, gen)
        self.body = body
    
    def toStr(self):
        """
        Return a string of the pdf object. This string can be included into
        a pdf file.
        
        """
        return self.getObjectString(self.body)


class PDFPath(PDFObject):
    """
    Class representing pathes to an entry in a pdf object (root of the object
    or entry in a dictionary or element of an array or any sequence of
    the later two). The path is stored in the member varialbe 'path' as a list
    of str objects (representing normalized names) and int object (representing
    0-based array indices/-1 refers to the position after the last element in
    the array; useful for appending elements to an array). An empty list
    represents a reference to the root of the object. ["/Foo", 3] for example
    refers to the 4th element in the array stored as the value to the key /Foo
    in the root dictionary.
    
    """
    def __init__(self, string):
        """
        Creates a new path object from a string. The created path does
        contain normalized pdfnames and integers as array indices.
        
        @type  string: A string containing a path to an entry of a pdf object.
                       The string must be the concatenation of
                       "/NotNormalizedValidPDFNames" and "[n]" index strings,
                       where n is a non-negative integer or * for the position
                       after the last element in the array. No whitespace
                       characters are allowed before or after the string or
                       between the single entries. An empty string refers to
                       the root of the object, a name to a key in a dictionary
                       and an index string to an element in an array (0-based).
                       Examples for valid strings are "/foo", "[3]", "[*]"
                       "/bar[5]/foo" or "[2]/bar" but not "foo", "[-3]", "[+3]",
                       "/foo  /bar" or "  [3]  ".
        
        """
        self.path = self.__subdividePath(string)
    
    def __normalizePath(self, path):
        """
        Takes a path to an entry in a pdf object and normalizes the single
        names in the path.
        
        @type  path: list of str/int
        @param path: A pdf path in which the /Names may not be normalized.
        
        @rtype:  list of str/int
        @return: A finished pdf path. All names in the path are normalized.
        
        """
        normPath = []
        for element in path:
            if isinstance(element, str):
                element = pdfutils.normalizePDFName(element)
            normPath.append(element)
        return normPath
    
    def __subdividePath(self, string):
        """
        Takes a pdf path string and converts it into a normalized pdf path.
        
        @type  string: str
        @param string: A valid pdf pathstring.
        
        @rtype: list of str/int
        @return: A finished pdf path.
        
        """
        utilities.checkType(string, str)
        namere  = re.compile(r"(?P<value>/[^/\[]*)", re.DOTALL)
        indexre = re.compile(r"\[(?P<value>[0-9]+|\*)\]")
        pathstr = string
        path = []
        while len(pathstr) > 0:
            found = namere.match(pathstr) or indexre.match(pathstr)
            if found:
                value = found.group("value")
                if value[0] != "/":
                    if value == "*":
                        value = -1
                    else:
                        value = int(value)
                path.append(value)
                pathstr = pathstr[len(found.group(0)):]
            else:
                raise PDFPathError, "pathstring %s is malformed" % string
        
        return self.__normalizePath(path)


class PDFStructObject(PDFObject):
    """
    A class representing pdf objects, whose contents is structured using a 1:1
    mapping of python structures to pdf structures.
    
    """
    def __init__(self, id, gen = 0):
        """
        Creates a new structured pdf object. The newly created object does not
        have any contents.
        
        @type  id: int
        @param id: The id of the pdf object.
        
        @type  gen: int
        @param gen: The generation of the pdf object.
        
        """
        PDFObject.__init__(self, id, gen)
        self.root = None
        self.appendix = ""
    
    def __applyIndexToList(self, index, alist):
        # convert -1 (resulting from a "[*]" token) into element after listend
        if index == -1:
            return len(alist)
        else:
            return index
    
    def __travPath(self, path, root):
        """
        Iterates along a pdf path in the pdf structured objects and returns
        the last element in the path. If the path does not exist within the
        pdf object, an exception is thrown.
        
        @type  path: list of str/int
        @param path: The path member of a PDFPath object.
        
        @type  root: any
        @param root: The root of the current subtree of the pdf structure.
        
        @rtype:  any
        @return: The value stored in the pdf object at the position referenced
                 by path.
        
        """
        # end of path reached
        if len(path) == 0:
            return root
        
        curEl, pathRemainder = path[0], path[1:]
        
        # found a /name (dictionary needed as root)?
        if isinstance(curEl, str):
            # is root a dict?
            if pdfutils.isPDFDict(root):
                if curEl in root:
                    return self.__travPath(pathRemainder, root[curEl])
        
        # was an [index] found (array needed as root)?
        elif isinstance(curEl, int):
            # is root an array?
            if pdfutils.isPDFArray(root):
                # [*] not allowed => convert -1 into an invalid element
                curEl = self.__applyIndexToList(curEl, root)
                if len(root) > curEl:
                    return self.__travPath(pathRemainder, root[curEl])
        
        raise PDFObjectError, "path does not match object structure"
    
    def __prepareNode(self, curEl, root):
        # was a /name found (dictionary needed as root)?
        if isinstance(curEl, str):
            # is root not a dict?
            if not pdfutils.isPDFDict(root):
                # then throw away old root and create a new dict
                root = pdfutils.makePDFDict()
            
            # is curEl not in root?
            if curEl not in root:
                # then create it
                root[curEl] = pdfutils.makePDFNull()
            
        # was an [index] found (array needed as root)?
        elif isinstance(curEl, int):
            # is root not an array?
            if not pdfutils.isPDFArray(root):
                # then throw away old root and create a new array
                root = pdfutils.makePDFArray()
            
            curEl = self.__applyIndexToList(curEl, root)
            # is curEl no valid index in root (list to short)?
            if len(root) <= curEl:
                # then append nulls to the list
                root += [pdfutils.makePDFNull()] * (curEl - len(root) + 1)
        
        # if this case is reached, something went wrong...
        else:
            raise Exception, "unreachable passage reached"
        
        return root
    
    def __createPath(self, path, root):
        """
        Iterates along a pdf path in a pdf structured objects and returns
        the last element in the path and the object generated. If the path does
        not exist within the pdf structure, it is created.
        
        """
        # end of path reached
        if len(path) == 1:
            root = self.__prepareNode(path[-1], root)
            return root, root
        
        curEl = path[0]
        
        # prepare root so that we can easily insert the right elements into it
        root = self.__prepareNode(curEl, root)
        
        # enter the next recursion
        result, root[curEl] = self.__createPath(path[1:], root[curEl])
        
        # we are finished, result contains the last element in the path and
        # root is the updated root passed to this function
        return result, root
    
    def __othersToStr(self, obj):
        if pdfutils.isPDFBool(obj):
            return pdfutils.serializePDFBool(obj)
        elif pdfutils.isPDFNull(obj):
            return pdfutils.serializePDFNull()
        elif pdfutils.isPDFNumber(obj):
            return pdfutils.serializePDFNumber(obj)
        elif pdfutils.isPDFName(obj):
            return pdfutils.serializePDFName(obj)
        elif pdfutils.isPDFString(obj):
            return pdfutils.serializePDFString(obj)
        elif pdfutils.isPDFRef(obj):
            return pdfutils.serializePDFRef(obj)
        elif obj is None:
            return ""
        else:
            raise PDFObjectError, "Object %s is of no valid pdf type" % \
                    repr(obj)
    
    def __arrayToStr(self, array, indent):
        newIndent = indent + 4
        newIndentStr = " " * newIndent
        indentStr = " " * indent
        
        result = "[\n"
        for value in array:
            result += newIndentStr
            if pdfutils.isPDFDict(value):
                result += self.__dictToStr(value, newIndent) + "\n"
            elif pdfutils.isPDFArray(value):
                result += self.__arrayToStr(value, newIndent) + "\n"
            else:
                result += self.__othersToStr(value) + "\n"
        
        result += indentStr + "]"
        return result
    
    def __dictToStr(self, dic, indent):
        keyIndent = indent + 4
        keyIndentStr = " " * keyIndent
        indentStr = " " * indent
        
        result = "<<\n"
        for key, value in dic.iteritems():
            if not pdfutils.isPDFName(key):
                raise PDFObjectError, ("key in pdf dictionary is no pdf" + \
                        " name but %s") % repr(key)
            
            keyStr = pdfutils.serializePDFName(key)
            result += keyIndentStr + keyStr + " "
            
            newIndent = keyIndent + len(keyStr) + len(" ")
            if pdfutils.isPDFDict(value):
                result += self.__dictToStr(value, newIndent) + "\n"
            elif pdfutils.isPDFArray(value):
                result += self.__arrayToStr(value, newIndent) + "\n"
            else:
                result += self.__othersToStr(value) + "\n"
        result += indentStr + ">>"
        return result
    
    def setAppendix(self, appendix):
        """
        Sets the appendix of the object. The appendix is inserted after the
        data contained in the object but before the endobj token. Normally, the
        appendix is used to add a file stream to the object.
        
        @type  appendix: str
        @param appendix: The appendix/filestream which should be added to the
        object.
        
        """
        self.appendix = appendix
    
    def generateBody(self):
        body = self.__serialize(self.root)
        return self.getObjectString(body + self.appendix)
    
    def toStr(self):
        return self.generateBody()
    
    def getObjectBody(self):
        return self.__serialize(self.root)
    
    def __serialize(self, root):
        if pdfutils.isPDFDict(root):
            return self.__dictToStr(root, 0) + "\n"
        elif pdfutils.isPDFArray(root):
            return self.__arrayToStr(root, 0) + "\n"
        else:
            return self.__othersToStr(root) + "\n"
    
    def exists(self, path):
        try:
            self.get(path)
            return True
        except PDFObjectError, e:
            return False
    
    def get(self, path):
        """
        """
        pathList = PDFPath(path).path
        result = self.__travPath(pathList, self.root)
        if result == None:
            raise PDFObjectError, "No value found at location '%s'" % path
        else:
            return result
    
    def set(self, path, value):
        path = PDFPath(path).path
        if len(path) == 0:
            self.root = value
        else:
            result, self.root = self.__createPath(path, self.root)
            curEl = path[-1]
            if curEl == -1:
                curEl = len(result)-1
            result = self.__prepareNode(curEl, result)
            result[curEl] = value
    
    def delete(self, path):
        errMsg = "could not delete object at location '%s'" % path
        pathList = PDFPath(path).path
        if len(pathList) == 0:
            if self.root != None:
                self.root = None
            else:
                raise PDFObjectError, errMsg
        else:
            lastEl = pathList[-1]
            result = self.__travPath(pathList[:-1], self.root)
            if pdfutils.isPDFDict(result) and isinstance(lastEl, str):
                if lastEl in result:
                    del result[lastEl]
                    return
            
            elif pdfutils.isPDFArray(result) and isinstance(lastEl, int):
                lastEl = self.__applyIndexToList(lastEl, result)
                if len(result) > lastEl:
                    del result[lastEl]
                    return
            
            raise PDFObjectError, errMsg


#obj = PDFStructObject(1, 0)

#print obj.toStr()

#print obj.get("")
#obj.set("", 3)
#print obj.root
#obj.delete("")
#print obj.root
#print obj.toStr()
#obj.set("/Foo", 3)
#obj.set("/Bar/Baz", 4)
#obj.set("/Moo[0]", 37)
#obj.set("/Moo[1]", 38)
#obj.set("/Moo[2]/Foo", 39)
#obj.set("/Koo[9]", pdfutils.makePDFName("ManyNullObjectsBeforeThisOne"))
#obj.set("/Koo[3]", pdfutils.makePDFName("Contains Spaces"))
#obj.set("/Baz[*]", 100)
#obj.set("/Baz[*]", 101)
#obj.set("/Baz[*]", 102)
#print obj.toStr()

#obj.get("/Koo").append(pdfutils.makePDFName("Appended/Object"))

#print obj.get("/Foo")
#print obj.get("/Moo[1]")
#print obj.get("/F#6F#6F") # same as /Foo
#print ""

#print obj.toStr()
#print ""
#print obj.root