"""
Module containing types representing pdf objects in python.

Type Mapping:
Each datatype in PDF has a corresponding datatype in python. The list below
shows the mapping of PDF to python datatypes.

PDF Datatype - Python Datatype:
    - Boolean values - bool
    - Integer and real numbers - int and float
    - Strings - str object, whose content is a valid pdf string
    - Names - str object, whose content is a valid pdf name
    - Arrays - list
    - Dictionaries - dict
    - Streams - Not Implemented
    - The null object - pdftypes.PDFNull
    - Object References - pdftypes.PDFRef



This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This application is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

$Id: pdftypes.py 1071 2008-05-25 18:58:08Z ahuesgen $
"""

class PDFNull(object):
    """
    Class representing the null type in pdf objects.
    
    """
    def __init__(self):
        """
        Creates a new pdf null object.
        
        """
        pass
    
    def __eq__(self, other):
        """
        Tests whether or not other is equal to self. All instances of PDFNull
        are equal to each other, no instance of any other type is equal to a
        PDFNull instance.
        
        @type  other: any
        @param other: the object, to which self is compared.
        
        @rtype:  bool
        @return: True if the test passes and False otherwise.
        
        """
        return isinstance(other, PDFNull)
    
    def __ne__(self, other):
        """
        Tests whether or not other is not equal to self.
        
        @type  other: any
        @param other: the object, to which self is compared.
        
        @rtype:  bool
        @return: True if the test passes and False otherwise.
        
        """
        return not self.__eq__(other)
    
    def toStr(self):
        """
        Returns a string representing a null object. The string can be written
        into a pdf file.
        
        @rtype:  str
        @return: A string representing a null object.
        
        """
        return "null"


class PDFRef(object):
    """
    Class representing references to pdf objects. Each reference has an id
    and a generation number uniquely identifying the object in refers to. A
    string containing the reference can be received with the method
    getObjectRef().
    
    """
    def __init__(self, id, gen=0):
        """
        Creates a new pdf reference object. The reference has an id and a
        generation
        
        @type  id: int
        @param id: The id of the pdf reference.
        
        @type  gen: int
        @param gen: The generation of the pdf reference.
        
        """
        self.id = id
        self.gen = gen
    
    def __eq__(self, other):
        """
        Tests whether or not other is equal to self. self is considered equal
        to other if other is an instance of PDFRef and the members id and gen
        are equal.
        
        @type  other: any
        @param other: the object, to which self is compared.
        
        @rtype:  bool
        @return: True if the test passes and False otherwise.
        
        """
        try:
            return isinstance(other, PDFRef) and self.id == other.id and \
                    self.gen == other.gen
        except Exception:
            return False
    
    def __ne__(self, other):
        """
        Tests whether or not other is not equal to self.
        
        @type  other: any
        @param other: the object, to which self is compared.
        
        @rtype:  bool
        @return: True if the test passes and False otherwise.
        
        """
        return not self.__eq__(other)
    
    def getObjectRef(self):
        """
        Returns a string containing a reference to the pdf object. This string
        can be included into a pdf file.
        
        @rtype:  str
        @return: A string representing a pdf reference.
        
        """
        return "%d %d R" % (self.id, self.gen)
