"""
Module containing various utility functions for working with vhist tags.



This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This application is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

$Id: tagutils.py 1071 2008-05-25 18:58:08Z ahuesgen $
"""

import re

def stringToTagstring(string):
    """
    Converts a string into a string, which can be included into a vhist tag.
    
    @type  string: unicode or str
    @param string: The string, which should be converted.
    
    @rtype:  str
    @return: A tag compatible utf8 string with \ and ] escaped.
    
    """
    if type(string) == unicode:
        string = string.encode("utf-8")
    
    string = string.replace("\\", "\\\\").replace("]", "\\]")
    return string.replace("\r\n", "\\n").replace("\n", "\\n").replace("\r", "\\n")

def tagstringToString(tagstring):
    """
    Converts a vhist tagstring into a normal utf8 encoded string.
    
    @type  tagstring: str
    @param tagstring: A vhist tagstring with \ and ] escaped.
    
    @rtype:  str
    @return: A normal python string (encoded in utf8)
    
    """
    origString = tagstring
    errMsg = "string '%s' is no valid tagstring"
    pos = 0
    while pos < len(tagstring):
        if tagstring[pos] == "\\":
            if tagstring[pos+1:pos+2] in ("\\", "]"):
                tagstring = tagstring[:pos] + tagstring[pos+1:]
            elif tagstring[pos+1:pos+2] == "n":
                tagstring = tagstring[:pos] + "\n" + tagstring[pos+2:]
            else:
                raise RuntimeError, errMsg % origString
        elif tagstring[pos] in "]\r\n":
            raise RuntimeError, errMsg % origString
        pos += 1
    
    return tagstring

# histtag datatypes
CSETS = {"str" : r"(([^\\\]]|(\\\\)|(\\\])|(\\n))*?)", \
         "int" : r"[0-9]*", \
         "md5" : r"(([a-f0-9]{32})?)"}


# int function
def convToInt(x):
    if x == "":
        return -1
    else:
        return int(x)

# conversion functions
CONV_FUNS = {"str" : tagstringToString, \
        "int" : convToInt, \
         #"int" : lambda x: ((x == "") and (-1)) or int(x), \
         "md5" : lambda x: x}


NAME = 0
TYPE = 1
GNAME = 2

class VHistTagParser(object):
    """
    A class representing a parser for vhist tags.
    """
    def __init__(self, tagname, *args):
        """
        Creates a new tagparser.
        
        @type  tagname: str
        @param tagname: The name of the vhist tag.
        
        @type  args: (str, str)
        @param args: a list of attribute name/type pairs. valid types are
                     "str", "int" and "md5". The name must consist of characters
                     from the set "{a-z,0-9,-}".
        
        """
        modArgs = [args[i] + ("g%d" % i,) for i in xrange(0, len(args))]
        
        self.reStr = self.__genTagRegex(tagname, *modArgs)
        self.tagAttrs = {}
        self.regex = re.compile(self.reStr, re.DOTALL)
        
        self.result = None
        self.startPos = -1
        self.endPos = -1
        
        for arg in modArgs:
            fun = CONV_FUNS[arg[TYPE]]
            self.tagAttrs[arg[NAME]] = (arg[GNAME], fun)
    
    def search(self, string, offset = 0):
        """
        Searches for the first occurence of the tagtype in a string.
        
        @type  string: str
        @param string: The string, in which the tag is searched.
        
        @type  offset: int
        @param offset: A offset, which is added to start and endpos of the
                       found tag (if string is a part of a larger string).
        
        @rtype:  bool
        @return: True if a tag was found and false otherwise.
        
        """
        result = self.regex.search(string)
        self.__fillResults(result, offset)
        return self.result != None
    
    def match(self, string, offset = 0):
        """
        Searches for the first occurence of the tagtype at the beginning of a
        string.
        
        @type  string: str
        @param string: The string, in which the tag is searched.
        
        @type  offset: int
        @param offset: A offset, which is added to start and endpos of the
                       found tag (if string is a part of a larger string).
        
        @rtype:  bool
        @return: True if a tag was found and false otherwise.
        
        """
        result = self.regex.match(string)
        self.__fillResults(result, offset) 
        return self.result != None
    
    def get(self, attribute):
        """
        Returns the value of given attribute in the last search. If no tag
        was found in the last search, None is returned.
        
        @type  attribute: str
        @param attribute: The name of the attribute.
        
        @rtype:  int or str
        @return: The value of the attribute. This value is already converted
                 to the correct datatype (integer, unescaped python string or
                 string containing the md5sum)
        
        """
        if self.result != None:
            grp, fun = self.tagAttrs[attribute]
            return fun(self.result.group(grp))
        else:
            return None
    
    def getStartPos(self):
        """
        Returns the position, at which the found tag starts.
        
        @rtype:  int
        @return: The start position of the found tag.
        
        """
        return self.startPos
    
    def getEndPos(self):
        """
        Returns the position, at which the found tag ends.
        
        @rtype:  int
        @return: The end position of the found tag.
        
        """
        return self.endPos
    
    def __fillResults(self, result, offset):
        self.result = result
        if result != None:
            self.startPos = offset + result.start()
            self.endPos = offset + result.end()
        else:
            self.startPos = self.endPos = -1
    
    def __genTagRegex(self, tagname, *args):
        regex = r"<--! \$" + tagname + "(.*?)"
        
        for arg in args:
            typedef = CSETS[arg[TYPE]]
            property = r"(?P<" + arg[GNAME] + r">" + typedef + r")"
            regex += r"\[" + arg[NAME] + r":" + property + r"\](.*?)"
        regex += r"-->"
        return regex
