"""
Module containing unittests for tagutils.py.



This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This application is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

$Id: test_cmdparser.py 1071 2008-05-25 18:58:08Z ahuesgen $
"""

import cmdparser
import unittest, verbosetestcase
import os

class TestCommandlineFileParser(verbosetestcase.VerboseTestCase):
    def setUp(self):
        self.oldEnv = os.environ
        os.environ = {}
        os.environ["VAR1"] = "Value1"
        os.environ["_COMPLEX_NAME_1"] = "complexValue"
        os.environ["_COMPLEX_CURLY"] = "curlyValue"
        os.environ["VAR_WITH_COMPLEX_VAL"] = "A complex value\n"
        os.environ["VAR_WITH_COMPLEX_VAL2"] = "C:\\Program Files\\Foo"
        os.environ["V1"] = "Val1"
        os.environ["V2"] = "Val2"
        os.environ["V3"] = "Val3"
    
    def tearDown(self):
        os.environ = self.oldEnv
    
    def testEscaping(self):
        data = [("", ""),
                ("NoEscapes", "NoEscapes"), ("Escaped Space", "Escaped\\ Space"),
                ("Newline\n", "Newline\\\n"), ("CarReturn\r", "CarReturn\\\r"),
                ("Tab\t", "Tab\\\t"), ("#Comment", "\\#Comment"),
                ("Backslash\\", "Backslash\\\\"), ("\"Quote\"", "\\\"Quote\\\""),
                ("'Quote'", "\\'Quote\\'"),
                ("Mixed  \t String \r\n 'A String'",
                 "Mixed\\ \\ \\\t\\ String\\ \\\r\\\n\\ \\'A\\ String\\'")]
        rule = cmdparser.ESCAPE_UNQUOTED
        for input, result in data:
            msg = "(%s, %s)" % (input, result)
            self.assertEquals(cmdparser.escape(input, rule), result, msg)

    def testReplaceVars(self):
        data = [("", ""), ("NoVars", "NoVars"), (r"\$FAKEVAR1", r"\$FAKEVAR1"),
                (r"\${FAKEVAR2}", r"\${FAKEVAR2}"),
                (r"\\\$FAKEVAR3", r"\\\$FAKEVAR3"),
                (r"\\\${FAKEVAR4}", r"\\\${FAKEVAR4}"),
                ("$NODEFINED", ""), ("${NOTDEFINED2}", ""),
                ("$VAR1", "Value1"), ("${VAR1}", "Value1"),
                ("$_COMPLEX_NAME_1", "complexValue"),
                ("${_COMPLEX_CURLY}", "curlyValue"),
                ("$001NoValidName", "$001NoValidName"),
                ("${001NoValidName}", "${001NoValidName}"),
                ("$ A Single Char", "$ A Single Char"),
                ("${Incomplete Name", "${Incomplete Name"),
                ("${Incomplete Name}", "${Incomplete Name}"),
                ("$VAR_WITH_COMPLEX_VAL", "A\\ complex\\ value\\\n"),
                ("$VAR_WITH_COMPLEX_VAL2", "C:\\\\Program\\ Files\\\\Foo"),
                ("Text $V1 more text ${V2} Backslashes\\\\ $V3 \\$NoVar",
                 "Text Val1 more text Val2 Backslashes\\\\ Val3 \\$NoVar")
                ]
        
        env = os.environ
        for input, result in data:
            msg = "(%s, %s)" % (input, result)
            self.assertEquals(cmdparser.handleEnvVars(input, env),
                    result, msg)


    def testSplitCommandlineSimple(self):
        data = [("", []),
                ("   ", []),
                ("OneArg", ["OneArg"]),
                ("'One String'", ["One String"]),
                ("No Variables", ["No", "Variables"]),
                ("More   Whitespace", ["More", "Whitespace"]),
                ("Escaped\\ Space", ["Escaped Space"]),
                ("1 2 3", ["1", "2", "3"]),
                ("# A comment", []),
                ("# A comment\nFoo Bar#Another Comment", ["Foo", "Bar"]),
                ("New\nLine", ["New", "Line"]),
                ("Contains\\\nNewline", ["Contains\nNewline"])
                ]
        
        for input, result in data:
            msg = "(%s, %s)" % (input, result)
            self.assertEquals(cmdparser.splitCommandline(input, True),
                    result, msg)
            self.assertEquals(cmdparser.splitCommandline(input, False),
                    result, msg)
   
    def testSplitCommandlineUseVars(self):
        data = [("$V1", ["Val1"]),
                ("$V1 ${V2}", ["Val1", "Val2"]),
                ("$V1${V2}", ["Val1Val2"]),
                ("$V1\n$V2", ["Val1", "Val2"]),
                ("Some Text With $V1", ["Some", "Text", "With", "Val1"]),
                ("Carni$V1", ["CarniVal1"]),
                ("${V1}nor", ["Val1nor"]),
                ("Pre ${V1} $V2 Post", ["Pre", "Val1", "Val2", "Post"]),
                ("Pre${V1}Post", ["PreVal1Post"]),
                ("$VAR_WITH_COMPLEX_VAL2", ["C:\\Program Files\\Foo"]),
                ("cd $VAR_WITH_COMPLEX_VAL2\\\\Bar", ["cd", 
                 "C:\\Program Files\\Foo\\Bar"]),
                ("cd \"$VAR_WITH_COMPLEX_VAL2 Bar\"", ["cd", 
                 "C:\\Program Files\\Foo Bar"]),
                ("single '$QUOTED' string", ["single", '$QUOTED', "string"]),
                ("single'$QUOTED'string", ["single$QUOTEDstring"])
                ]
        
        for input, result in data:
            msg = "(%s, %s)" % (input, result)
            self.assertEquals(cmdparser.splitCommandline(input, True),
                    result, msg)
    
    def testSplitCommandlineWithoutVars(self):
        data = [("$V1", ["$V1"]),
                ("'$V1'", ["$V1"]),
                ("\"$V1\"", ["$V1"]),
                ("$V1 \"$V2\" '${V3}'", ["$V1", "$V2", "${V3}"]),
                ("# comment\n \"Hello'\\\\\"", ["Hello'\\"]),
                ("foo \# bar", ["foo", "#", "bar"]),
                ("hello # world", ["hello"]),
                ("'\\\\'", ["\\\\"])]
        
        for input, result in data:
            msg = "(%s, %s)" % (input, result)
            self.assertEquals(cmdparser.splitCommandline(input, False),
                    result, msg)
    
    def testShlexAssumptions(self):
        pass

if __name__ == "__main__":
    unittest.main()


