"""
Module containing unittests for fileiteratior.py



This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This application is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

$Id: test_fileiterator.py 1071 2008-05-25 18:58:08Z ahuesgen $
"""

import fileiterator
import unittest, verbosetestcase


class MockFile(object):
    def __init__(self, string):
        self.string = string
        self.lst = string.splitlines(True)
        self.pos = 0
    
    def read(self, numBytes):
        return self.string[self.pos:self.pos + numBytes]
    
    def seek(self, pos, relativeTo = 0):
        if relativeTo == 0:
            self.pos = pos
        elif relativeTo == 1:
            self.pos += pos
        elif relativeTo == 2:
            self.pos = len(self.string) + pos
        else:
            raise RuntimeError, "invalid 2nd argument to seek"
    
    def tell(self):
        return self.pos

class TestFileIterator(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    def testEmpty(self):
        mock = MockFile("")
        it = fileiterator.FileIterator(mock)
        it.moveToBegin()
        self.assertEquals(it.next(), None)
        
        it.moveToEnd()
        self.assertEquals(it.prev(), None)
    
    def testLinesNext(self):
        string = "foo\nbar\nmoo\nkoo\n"
        mock = MockFile(string)
        it = fileiterator.FileIterator(mock)
        it.moveToBegin()
        
        lst = []
        for i in xrange(0, 4):
            x = it.next()
            lst.append(x)
        result = "".join(lst) 
        self.assertEquals(result, string)
    
    def testLinesPrev(self):
        string = "foo\nbar\nmoo\nkoo\n"
        mock = MockFile(string)
        it = fileiterator.FileIterator(mock)
        it.moveToEnd()
        
        lst = []
        for i in xrange(0, 4):
            x = it.prev()
            lst = [x] + lst
        result = "".join(lst) 
        self.assertEquals(result, string)
    
    def testNextPrev1(self):
        string = "foo\nbar\nmoo\nkoo\n"
        operations = [+1, +1, -1, +1, +1, -1, +1, -1, -1]
        
        mock = MockFile(string)
        it = fileiterator.FileIterator(mock)
        it.moveToBegin()
        pos = -1
        
        for op in operations:
            if op == +1:
                x = it.next()
            else:
                x = it.prev()
            pos += op
            self.assertEquals(mock.lst[pos], x, str(op))
            
    def testNextPrev2(self):
        string = "foo\nbar\nmoo\nkoo\n"
        operations = [-1, -1,+1, -1, -1, +1, -1, +1, +1]
        
        mock = MockFile(string)
        it = fileiterator.FileIterator(mock)
        it.moveToEnd()
        pos = len(mock.lst)
        
        for op in operations:
            if op == +1:
                x = it.next()
            else:
                x = it.prev()
            pos += op
            self.assertEquals(mock.lst[pos], x, str(op))
    
    def testMoveBeyondEnd(self):
        string = "foo\nbar\nmoo\nkoo\n"
        mock = MockFile(string)
        it = fileiterator.FileIterator(mock)
        it.moveToEnd()
        self.assertEquals(it.next(), None)
    
    def testMoveBeyondBegin(self):
        string = "foo\nbar\nmoo\nkoo\n"
        mock = MockFile(string)
        it = fileiterator.FileIterator(mock)
        it.moveToBegin()
        self.assertEquals(it.prev(), None)
    
    def testWindowsNewline(self):
        string = "foo\r\nbar\r\nmoo"
        mock = MockFile(string)
        it = fileiterator.FileIterator(mock)
        it.moveToBegin()
        
        lst = []
        for i in xrange(0, 3):
            x = it.next()
            lst.append(x)
        result = "".join(lst) 
        self.assertEquals(result, string)
    
    def testLongLine(self):
        string = "A long line which does not fit into the default buffer\n"
        mock = MockFile(string)
        it = fileiterator.FileIterator(mock, False, 10)
        it.moveToBegin()
        
        result = it.next()
        self.assertEquals(result, string)
    
    def testLongLine2(self):
        string = "A long line which does not fit into the default buffer\n"
        mock = MockFile(string)
        it = fileiterator.FileIterator(mock, False, 10)
        it.moveToEnd()
        
        result = it.prev()
        self.assertEquals(result, string)
    
    def testTooLongLine(self):
        string = "A line, which exceeds the max buffersize allowed\n"
        mock = MockFile(string)
        it = fileiterator.FileIterator(mock, False, 10, 20)
        it.moveToBegin()
        
        self.assertRaises(MemoryError, it.next)
    
    def testTooLongLine(self):
        string = "A line, which exceeds the max buffersize allowed\n"
        mock = MockFile(string)
        it = fileiterator.FileIterator(mock, False, 10, 20)
        it.moveToEnd()
        
        self.assertRaises(MemoryError, it.prev)
    
    def testLongLines(self):
        string1 = "This is one long line as far as I can see\n"
        string2 = "This is another long line which is even longer\n"
        string3 = "This is a behemoth of a line and does not have a \\n"
        
        string = string1 + string2 + string3
        mock = MockFile(string)
        it = fileiterator.FileIterator(mock, False, 10)
        it.moveToBegin()
        
        for line in mock.lst:
            result = it.next()
            self.assertEquals(line, result)
    
    def testAddPosition(self):
        string = "Line1\nLine2\nLine3\nLine4\n"
        mock = MockFile(string)
        it = fileiterator.FileIterator(mock, True)
        it.moveToBegin()
        
        pos = 0
        for line in mock.lst:
            result = it.next()
            self.assertEquals(result[0], line)
            self.assertEquals(result[1], pos)
            pos += len(line)
    
    def testAddPosition2(self):
        string = "Line1\nLine2\nLine3\nLine4\n"
        mock = MockFile(string)
        it = fileiterator.FileIterator(mock, True)
        it.moveToEnd()
        
        pos = len(string)
        for line in mock.lst[::-1]:
            pos -= len(line)
            result = it.prev()
            self.assertEquals(result[0], line)
            self.assertEquals(result[1], pos)


if __name__ == "__main__":
    unittest.main()
