"""
Module containing unittests for pdfobjs.py.



This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This application is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

$Id: test_pdfobjs.py 1071 2008-05-25 18:58:08Z ahuesgen $
"""

import pdfobjs, pdftypes
import unittest, verbosetestcase

class TestPDFPath(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    def testCreation(self):
        testData = [
                ("", []),
                ("/", ["/"]),
                ("/foo", ["/foo"]),
                ("[0]", [0]),
                ("[234]", [234]),
                ("[*]", [-1]),
                ("/f#6F#6F", ["/foo"]),
                ("/f\xFF\xFE", ["/f#ff#fe"]),
                ("/foo/bar", ["/foo", "/bar"]),
                ("[1][2]", [1, 2]),
                ("/foo[1]", ["/foo", 1]),
                ("[1]/foo", [1, "/foo"]),
                ("[1]/foo[2]", [1, "/foo", 2]),
                ("/foo[10]/bar", ["/foo", 10, "/bar"])
                   ]
        
        for value, expected in testData:
            msg = str((value, expected))
            result = pdfobjs.PDFPath(value)
            self.assertEquals(result.path, expected, msg)


class TestPDFObject(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    def testCreation(self):
        testData = [(1, 0), (2, 0)]
        
        for id, gen in testData:
            msg = str((id, gen))
            obj = pdfobjs.PDFStructObject(id, gen)
            self.assertEquals(obj.id, id, msg)
            self.assertEquals(obj.gen, gen, msg)


class TestPDFStructObject(verbosetestcase.VerboseTestCase):
    def setUp(self):
        self.obj = pdfobjs.PDFStructObject(0, 1)
    
    def tearDown(self):
        pass
    
    def testGetSetExists(self):
        null = pdftypes.PDFNull()
        
        testData = [
                ("", "/foo"),
                ("", {}),
                ("", []),
                ("/foo", "/bar"),
                ("[0]", "/bar"),
                ("[1]", "/bar"),
                ("/foo/bar", "/moo"),
                ("[1][0]", "/foo"),
                ("/foo[0]", "/bar"),
                ("[2]/foo", "/bar")
                   ]
        
        for path, value in testData:
            msg = str((path, value))
            obj = pdfobjs.PDFStructObject(1, 0)
            obj.set(path, value)
            self.assertEquals(obj.exists(path), True, msg)
            self.assertEquals(obj.get(path), value, msg)
    
    def testGetSetExistsSpecialCases(self):
        testData = [
                ("[*]", "[0]", "/foo"),
                ("/foo[*]", "/foo[0]", "/foo"),
                ("[*]/foo", "[0]/foo", "/foo")
                   ]
        
        for pathSet, pathGet, value in testData:
            msg = str((pathSet, pathGet, value))
            diagStr = str((pathSet, pathGet, value))
            obj = pdfobjs.PDFStructObject(1, 0)
            obj.set(pathSet, value)
            self.assertEquals(obj.exists(pathGet), True, msg)
            self.assertEquals(obj.get(pathGet), value, msg)
    
    def testEmptyGet(self):
        self.assertEquals(self.obj.exists(""), False)
        self.assertRaises(pdfobjs.PDFObjectError, self.obj.get, "")
    
    def testDelete(self):
        testData = ["", "/foo", "[0]", "/foo[0]", "[0]/foo"]
        
        for path in testData:
            msg = path
            obj = pdfobjs.PDFStructObject(1, 0)
            obj.set(path, "/foo")
            self.assertEquals(obj.exists(path), True, msg)
            obj.delete(path)
            self.assertEquals(obj.exists(path), False, msg)
    
    def testEmptyDelete(self):
        testData = ["", "/foo", "[0]", "/foo[0]", "[0]/foo", "[*]"]
        
        for path in testData:
            msg = path
            self.assertRaises(pdfobjs.PDFObjectError, self.obj.delete, msg)

if __name__ == "__main__":
    unittest.main()
