"""
Module containing unittests for pdfutils.py.



This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This application is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

$Id: test_pdfutils.py 1071 2008-05-25 18:58:08Z ahuesgen $
"""

import pdfutils, pdftypes
import datetime, unittest, verbosetestcase


class TestToPDFDate(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    def testConversion(self):
        dt = datetime.datetime
        
        # create and test some date string tuples
        testDates = [
            (dt(2000, 10, 11, 23, 5, 3), "(D:20001011230503)"),
            (dt(1234, 1, 2, 3, 4, 5), "(D:12340102030405)"),
            (dt(1, 12, 24, 20, 15, 0), "(D:00011224201500)")
            ]
        
        for datepair in testDates:
            msg = str(datepair)
            self.assertEqual(pdfutils.toPDFDate(datepair[0]), datepair[1], msg)
    
    def testException(self):
        # make sure, that an exception is risen when the passed parameter is
        # incompatible
        self.assertRaises(Exception, pdfutils.toPDFDate, "foo")
        self.assertRaises(Exception, pdfutils.toPDFDate, 123)
        self.assertRaises(Exception, pdfutils.toPDFDate, [])


class TestMakePDFBool(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    def testString(self):
        self.assertEquals(pdfutils.makePDFBool("true"), True)
        self.assertEquals(pdfutils.makePDFBool("false"), False)
    
    def testBool(self):
        self.assertEquals(pdfutils.makePDFBool(True), True)
        self.assertEquals(pdfutils.makePDFBool(False), False)
    
    def testRaises(self):
        self.assertRaises(pdfutils.ConversionError, pdfutils.makePDFBool,
                "wrong value")
        self.assertRaises(TypeError, pdfutils.makePDFBool, 123)


class TestMakePDFNumber(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDows(self):
        pass
    
    def testString(self):
        self.assertEquals(pdfutils.makePDFNumber("123"), 123)
        self.assertEquals(pdfutils.makePDFNumber("0"), 0)
        self.assertEquals(pdfutils.makePDFNumber("-123"), -123)
        self.assertAlmostEquals(pdfutils.makePDFNumber("1.12"), 1.12, 5)
        self.assertAlmostEquals(pdfutils.makePDFNumber("-1.12"), -1.12, 5)
        self.assertAlmostEquals(pdfutils.makePDFNumber("0.0"), 0.0, 5)
        self.assertAlmostEquals(pdfutils.makePDFNumber("1."), 1.0, 5)
    
    def testNumber(self):
        numbers = [0, 1, 2, 3, -1, -2, -3, 0.0, -1.0, 1.1234]
        
        for num in numbers:
            self.assertEquals(pdfutils.makePDFNumber(num), num)
    
    def testRaises(self):
        self.assertRaises(pdfutils.ConversionError, pdfutils.makePDFNumber,
                "wrong value")
        self.assertRaises(TypeError, pdfutils.makePDFNumber, None)


class TestMakePDFString(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    def testString(self):
        pass


class TestMakePDFArray(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    def testReturn(self):
        self.assertEquals(pdfutils.makePDFArray(), [])


class TestMakePDFDict(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    def testReturn(self):
        self.assertEquals(pdfutils.makePDFDict(), {})


class TestMakePDFNull(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    def testReturn(self):
        null = pdftypes.PDFNull()
        self.assertEquals(pdfutils.makePDFNull(), null)


class TestMakePDFRef(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    def testInts(self):
        data = [(1, 0), (100, 10), (123, 3)]
        for id, gen in data:
            ref = pdfutils.makePDFRef(id, gen)
            self.assert_(isinstance(ref, pdftypes.PDFRef))
            self.assertEquals(ref.id, id)
            self.assertEquals(ref.gen, gen)


class TestIsPDFBool(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    def testValues(self):
        self.assertEquals(pdfutils.isPDFBool(True), True)
        self.assertEquals(pdfutils.isPDFBool(False), True)
        self.assertEquals(pdfutils.isPDFBool("True"), False)
        self.assertEquals(pdfutils.isPDFBool(0), False)
        self.assertEquals(pdfutils.isPDFBool(1.0), False)


class TestIsPDFNumber(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    def testValues(self):
        self.assertEquals(pdfutils.isPDFNumber(0), True)
        self.assertEquals(pdfutils.isPDFNumber(1), True)
        self.assertEquals(pdfutils.isPDFNumber(-1), True)
        self.assertEquals(pdfutils.isPDFNumber(0.0), True)
        self.assertEquals(pdfutils.isPDFNumber(-1.23), True)
        self.assertEquals(pdfutils.isPDFNumber("123"), False)
        self.assertEquals(pdfutils.isPDFNumber(True), False)


class TestIsPDFName(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    def testStrings(self):
        self.assertEquals(pdfutils.isPDFName("/Foo"), True)
        self.assertEquals(pdfutils.isPDFName("/Bar#12"), True)
        self.assertEquals(pdfutils.isPDFName("/Baz$"), True)
        self.assertEquals(pdfutils.isPDFName("/"), True)
        self.assertEquals(pdfutils.isPDFName(u"/Foo"), True)
        self.assertEquals(pdfutils.isPDFName("ABC"), False)
        self.assertEquals(pdfutils.isPDFName("///"), False)
    
    def testOthers(self):
        self.assertEquals(pdfutils.isPDFName(123), False)
        self.assertEquals(pdfutils.isPDFName(True), False)


class TestToPDFString(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass    
    
    def testConversion(self):
        # create and test some test data
        testData = [
            ("foobar", "(foobar)"),
            ("\n\t\r\b\f", "(\\n\\t\\r\\b\\f)"),
            ("(Hello World)", "(\\(Hello World\\))"),
            ("\\foo/bar", "(\\\\foo/bar)"),
            ("\x01\x02\x03\xff\xee\xdd", "(\x01\x02\x03\xff\xee\xdd)")
            ]
        
        for datapair in testData:
            msg = str(datapair)
            self.assertEquals(pdfutils.toPDFString(datapair[0]), datapair[1],
                    msg)


class TestToPDFArray(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass    
    
    def testConversion(self):
        testData = [
            (["foo", "bar"], "[foo bar]"),
            (["a", "b", "c"], "[a b c]"),
            ([], "[]"),
            (["a"], "[a]")
            ]
        
        for datapair in testData:
            msg = str(datapair)
            self.assertEquals(pdfutils.toPDFArray(datapair[0]), datapair[1],
                    msg)


class TestToPDFName(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass    
    
    def testConversion(self):
        testData = [
            ("", "/"),
            ("foo", "/foo"),
            ("%/ \n\r#", "/#25#2f#20#0a#0d#23"),
            ("\x00\x01\xDD\xEE", "/#00#01#dd#ee")
            ]
        
        for datapair in testData:
            msg = str(datapair)
            self.assertEquals(pdfutils.toPDFName(datapair[0]), datapair[1], msg)


class TestFromPDFName(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass    
    
    def testConversion(self):
        testData = [
            ("/", ""),
            ("/foo", "foo"),
            ("/#25#2f#20#0a#0d#23", "%/ \n\r#"),
            ("/#25#2F#20#0A#0D#23", "%/ \n\r#"),
            ("/#00#01#dd#ee", "\x00\x01\xDD\xEE")
            ]
        
        for datapair in testData:
            msg = str(datapair)
            self.assertEquals(pdfutils.fromPDFName(datapair[0]), datapair[1],
                    msg)

    def testRaises(self):
        testData = ["", "foo", "//", "/%", "/#xy", "/#"]
        
        for value in testData:
            self.assertRaises(RuntimeError, pdfutils.normalizePDFName, value)

class TestNormalizePDFName(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass    
    
    def testConversion(self):
        testData = [
            ("/", "/"),
            ("/#37", "/7"),
            ("/\xab", "/#ab"),
            ("/#ab", "/#ab"),
            ("/#AB", "/#ab"),
            ("/Name", "/Name"),
            ("/Foo\xab", "/Foo#ab"),
            ("/The#37Foos", "/The7Foos")
            ]
        
        for datapair in testData:
            msg = str((datapair))
            self.assertEquals(pdfutils.normalizePDFName(datapair[0]),
                    datapair[1], msg)
    
    def testRaises(self):
        testData = ["", "foo", "//", "/%", "/#xy", "/#"]
        
        for value in testData:
            self.assertRaises(RuntimeError, pdfutils.normalizePDFName, value)


class TestUtfifyString(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass    
    
    def testConversionUTF8(self):
        testData = [
            ("", ""),
            ("foobar", "foobar"),
            (u"foobar", "foobar"),
            (u"\uabcd", u"\uabcd".encode("utf-8")),
            (u"\uabcd".encode("utf-8"), u"\uabcd".encode("utf-8"))
            ]
        
        for datapair in testData:
            msg = str(datapair)
            self.assertEquals(pdfutils.utf8ifyString((datapair[0])),
                    datapair[1], msg)
    
    def testConversionUTF16(self):
        testData = [
            ("", "".encode("utf-16-be")),
            ("foobar", "foobar".encode("utf-16-be")),
            (u"foobar", "foobar".encode("utf-16-be")),
            (u"\uabcd", u"\uabcd".encode("utf-16-be")),
            (u"\uabcd".encode("utf-8"), u"\uabcd".encode("utf-16-be"))
            ]
        
        for datapair in testData:
            msg = str(datapair)
            self.assertEquals(pdfutils.utf16ifyString((datapair[0])),
                    datapair[1], msg)


class TestMakeTagCompatible(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass    
    
    def testConversion(self):
        testData = [
            ("", ""),
            ("foobar", "foobar"),
            ("foo\\bar", "foo\\\\bar"),
            ("foo]bar", "foo\\]bar"),
            ("\\", "\\\\"),
            ("]", "\\]")
            ]
        for datapair in testData:
            msg = str(datapair)
            self.assertEquals(pdfutils.makeTagCompatibleString(datapair[0]),
                    datapair[1], msg)


if __name__ == "__main__":
    unittest.main()
