"""
Module containing unittests for tagutils.py.



This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This application is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

$Id: test_tagutils.py 1071 2008-05-25 18:58:08Z ahuesgen $
"""

import tagutils
import unittest, verbosetestcase


class TestConvertTagStrings(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    def testToTagString(self):
        data = [("simple string", "simple string"), ("[bracket", "[bracket"),
                ("bracket2]", "bracket2\\]"), ("slash\\", "slash\\\\")]
        
        for input, result in data:
            msg = "(%s, %s)" % (input, result)
            self.assertEquals(tagutils.stringToTagstring(input), result, msg)
    
    #def testToTagStringRaises(self):
        #self.assertRaises(RuntimeError, tagutils.stringToTagstring, "\n")
        #self.assertRaises(RuntimeError, tagutils.stringToTagstring, "\r")
    
    def testFromTagString(self):
        data = [("simple string", "simple string"), ("[bracket", "[bracket"),
                ("bracket2\\]", "bracket2]"), ("backslash\\\\",
                "backslash\\")]
        
        for input, result in data:
            msg = "(%s, %s)" % (input, result)
            self.assertEquals(tagutils.tagstringToString(input), result, msg)
    
    def testFromTagStringRaises(self):
        self.assertRaises(RuntimeError, tagutils.tagstringToString, "xyz\n")
        self.assertRaises(RuntimeError, tagutils.tagstringToString, "xyz\r")
        self.assertRaises(RuntimeError, tagutils.tagstringToString, "xyz]")


class TestTagParser(verbosetestcase.VerboseTestCase):
    def setUp(self):
        pass
    
    def tearDown(self):
        pass
    
    def testSearchOneInt(self):
        parse = tagutils.VHistTagParser("FOO", ("key1", "int"))
        tagstring = "<--! $FOO [key1:23]-->"
        self.assertEquals(parse.search(tagstring), True)
        self.assertEquals(parse.get("key1"), 23)
        self.assertEquals(parse.getStartPos(), 0)
        self.assertEquals(parse.getEndPos(), len(tagstring))
        
        self.assertEquals(parse.match(tagstring), True)
        self.assertEquals(parse.get("key1"), 23)
        self.assertEquals(parse.getStartPos(), 0)
        self.assertEquals(parse.getEndPos(), len(tagstring))
    
    def testSearchOneString(self):
        parse = tagutils.VHistTagParser("FOO", ("key1", "str"))
        tagstring = "<--! $FOO [key1:abc]-->"
        self.assertEquals(parse.search(tagstring), True)
        self.assertEquals(parse.get("key1"), "abc")
        self.assertEquals(parse.getStartPos(), 0)
        self.assertEquals(parse.getEndPos(), len(tagstring))
        
        self.assertEquals(parse.match(tagstring), True)
        self.assertEquals(parse.get("key1"), "abc")
        self.assertEquals(parse.getStartPos(), 0)
        self.assertEquals(parse.getEndPos(), len(tagstring))
            
    def testSearchOneString2(self):
        parse = tagutils.VHistTagParser("FOO", ("key1", "str"))
        tagstring = "<--! $FOO [key1:123]-->"
        self.assertEquals(parse.search(tagstring), True)
        self.assertEquals(parse.get("key1"), "123")
        self.assertEquals(parse.getStartPos(), 0)
        self.assertEquals(parse.getEndPos(), len(tagstring))
        
        self.assertEquals(parse.match(tagstring), True)
        self.assertEquals(parse.get("key1"), "123")
        self.assertEquals(parse.getStartPos(), 0)
        self.assertEquals(parse.getEndPos(), len(tagstring))
    
    def testSearchOneEscapedString(self):
        parse = tagutils.VHistTagParser("FOO", ("key1", "str"))
        tagstring = "<--! $FOO [key1:[foo\\]backslash\\\\]-->"
        self.assertEquals(parse.search(tagstring), True)
        self.assertEquals(parse.get("key1"), "[foo]backslash\\")
        self.assertEquals(parse.getStartPos(), 0)
        self.assertEquals(parse.getEndPos(), len(tagstring))
        
        self.assertEquals(parse.match(tagstring), True)
        self.assertEquals(parse.get("key1"), "[foo]backslash\\")
    
    def testSearchOneMD5(self):
        parse = tagutils.VHistTagParser("FOO", ("key1", "md5"))
        tagstring = "<--! $FOO [key1:1234567890abcdeffedcba0987654321]-->"
        self.assertEquals(parse.search(tagstring), True)
        self.assertEquals(parse.get("key1"), "1234567890abcdeffedcba0987654321")
        
        self.assertEquals(parse.match(tagstring), True)
        self.assertEquals(parse.get("key1"), "1234567890abcdeffedcba0987654321")
    
    def testSearchTwoProperties(self):
        parse = tagutils.VHistTagParser("FOO", ("key1", "str"), ("key2", "int"))
        tagstring = "<--! $FOO [key1:hello world][key2:1234567]-->"
        self.assertEquals(parse.search(tagstring), True)
        self.assertEquals(parse.get("key1"), "hello world")
        self.assertEquals(parse.get("key2"), 1234567)
        
        self.assertEquals(parse.match(tagstring), True)
        self.assertEquals(parse.get("key1"), "hello world")
        self.assertEquals(parse.get("key2"), 1234567)
    
    def testSearchAdditionalProperites(self):
        parse = tagutils.VHistTagParser("FOO", ("key1", "str"), ("key2", "int"))
        tagstring = "<--! $FOO [key0:foo] [key1:hello world] [keyHalf:bar] [key2:1234567] [key3:baz]-->"
        self.assertEquals(parse.search(tagstring), True)
        self.assertEquals(parse.get("key1"), "hello world")
        self.assertEquals(parse.get("key2"), 1234567)
        
        self.assertEquals(parse.match(tagstring), True)
        self.assertEquals(parse.get("key1"), "hello world")
        self.assertEquals(parse.get("key2"), 1234567)
    
    def testSearchTagInMiddle(self):
        parse = tagutils.VHistTagParser("FOO", ("key1", "str"))
        tagstring = "%other data ... <--! $FOO [key1:foo]-->"
        self.assertEquals(parse.search(tagstring), True)
        self.assertEquals(parse.get("key1"), "foo")
        
        self.assertEquals(parse.match(tagstring), False)
        self.assertEquals(parse.get("key1"), None)
    
    def testInvalidInitialization(self):
        pass #self.assertRaises()

if __name__ == "__main__":
    unittest.main()

