"""
Module containing various utility functions for creating and working with
pdf related objects in python.



This file is part of the VHIST project, http://www.nf.mpg.de/vhist,
which defines a file format suitable for documenting workflows. VHIST
files can store meta-data about the workflow and allow to embed
arbitrary binary data, such as images or log files, inside the
documentation. Embedded data can be validated using multiple facilities.
VHIST files can be viewed with any PDF browser.

(c) Max-Planck-Institute for Neurological Research (MPIFNF)
Cologne, Germany 2005 - 2013
Email: vhist@nf.mpg.de, http://www.nf.mpg.de/vhist

This application is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation
(LGPL version 2.1):

This application is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this application. If not, see
<http://www.gnu.org/licenses/>.

$Id: validatetag.py 1896 2009-11-16 18:32:48Z ahuesgen $
"""

import re, zlib, os
import checksums
import logger

# histtag datatypes
CSETS = {"str" : r"(([^\\\]]|(\\\\)|(\\\])|(\\n))*?)", \
         "int" : r"[0-9]*", \
         "md5" : r"(([a-f0-9]{32})?)"}


def genTagRegex(tagname, *args):
    regex = r"<--! \$" + tagname + "(.*?)"
    
    for arg in args:
        typedef = CSETS[arg[2]]
        property = r"(?P<" + arg[1] + r">" + typedef + r")"
        regex += r"\[" + arg[0] + r":" + property + r"\](.*?)" 
    regex += r"-->"
    return regex


SECTION_REGSTR = \
genTagRegex("VHIST_SECTION", ("version", "ver", "str"),
        ("creator", "cre", "str"), ("title", "title", "str"),
        ("left", "left", "int"), ("size", "size", "int"),
        ("index", "index", "int"), ("md5section", "md5", "md5"),
        ("previousmd5", "pmd5", "md5"), ("previousmarker", "pmark", "int"))

FILETAG_PROPERTIES = (("filetype", "ft", "str"), ("filename", "fn", "str"), 
        ("desc", "desc", "str"), ("comment", "comment", "str"),
        ("compression", "comp", "str"), ("filesize", "fsize", "int"),
        ("cfilesize", "csize", "int"), ("blocksize", "bsize", "int"),
        ("offset", "offset", "int"), ("md5file", "md5", "md5"),
        ("md5cfile", "cmd5", "md5"))

FILEBEGIN_REGSTR = genTagRegex("VHIST_EMBEDDEDFILE_BEGIN", *FILETAG_PROPERTIES)
FILEEND_REGSTR = genTagRegex("VHIST_EMBEDDEDFILE_END", *FILETAG_PROPERTIES)

SECTION_RE   = re.compile(SECTION_REGSTR, re.DOTALL)
FILEBEGIN_RE = re.compile(FILEBEGIN_REGSTR, re.DOTALL)
FILEEND_RE   = re.compile(FILEEND_REGSTR, re.DOTALL)

SECTION_TAG_START = "<--! $VHIST_SECTION"

#---------------------------------------------
# Section validation
#---------------------------------------------

def parseSection(data, startpos):
    """Try to parse the section at the given position and return a dictionary
       containing the section's information. If no section is found at the
       given position, None is returned.
       
       keyword arguments:
       startpos - The position, at which the section starts
       data     - The string containing the section
    """
    found = SECTION_RE.match(data, startpos)
    if found != None:
        result = {}
        result["version"]      = found.group("ver")
        result["creator"]      = found.group("cre")
        result["title"]        = found.group("title")
        result["left"]         = int(found.group("left"))
        result["size"]         = int(found.group("size"))
        result["index"]        = int(found.group("index"))
        result["md5"]          = found.group("md5")
        result["prevmd5"]      = found.group("pmd5")
        result["prevmark"]   = -1
        if found.group("pmark") != "":
            result["prevmark"]     = int(found.group("pmark"))
        result["tagstart"]     = found.start()
        result["sectionstart"] = result["tagstart"] - result["left"]
        result["sectionend"]   = result["sectionstart"] + result["size"]
        
        tagStart = found.start(0)
        md5Start = found.start("md5") - tagStart
        md5End = md5Start + len(found.group("md5"))
        tag1 = found.group(0)[:md5Start]
        tag2 = found.group(0)[md5End:]
        
        # create tag which is used for validation (checksum reset to 0)
        result["modtag"]       = tag1 + "0"*32 + tag2
        
        return result
    else:
        return None


def findLastSection(data, endpos):
    # search for section
    curPos = endpos
    while curPos != -1:
        pos = data.rfind(SECTION_TAG_START, 0, curPos)
        if pos != -1:
            found = parseSection(data, pos)
            if found != None:
                return found
        curPos = pos
    
    return None


def calcSectionMD5(data, tagInfo):
    start1 = tagInfo["sectionstart"]
    end1   = tagInfo["tagstart"]
    start2 = tagInfo["tagstart"] + len(tagInfo["modtag"])
    end2   = tagInfo["sectionend"]
    
    md5sum = checksums.md5(data[start1:end1])
    md5sum.update(tagInfo["modtag"])
    md5sum.update(data[start2:end2])
    return md5sum


def validateSection(data, tagInfo, nextInfo):
    result = {"correctIndex": False, "correctPMD5": False, "correctMD5": False}
    # print title
    logger.log(3, "Section: '%(title)s' found" % tagInfo)
    
    # print index
    result["index"] = tagInfo["index"]
    if nextInfo != None and tagInfo["index"] != nextInfo["index"] - 1:
        values = (tagInfo["index"], nextInfo["index"]-1)
        logger.error("section index is %d but was supposed to be %d" % values)
    else:
        logger.log(4, "section index is %d: ok", tagInfo["index"])
        result["correctIndex"] = True
    
    # print version and creator
    logger.log(4, "version is %(version)s" % tagInfo)
    logger.log(4, "creator is %(creator)s" % tagInfo)
    
    # print md5
    logger.log(4, "md5 is                 %s", tagInfo["md5"])
    
    # print previous md5 of next tag
    if nextInfo == None:
        nextMD5 = "not available"
    else:
        nextMD5 = nextInfo["prevmd5"]
    
    logger.log(4, "pmd5 of next secton is %s", nextMD5)
    
    if nextInfo != None and tagInfo["md5"] != nextInfo["prevmd5"]:
        logger.error("md5 and pmd5 of next section differ")
    else:
        result["correctPMD5"] = True
    
    # generate and print real md5
    md5sum = calcSectionMD5(data, tagInfo)
    hexmd5 = md5sum.hexdigest()
    logger.log(4, "generated md5 is       %s", hexmd5)
    if hexmd5 != tagInfo["md5"]:
        logger.error("md5 incorrect")
    else:
        logger.log(4, "md5 correct")
        result["correctMD5"] = True
    
    logger.log(1, "")
    return result


#---------------------------------------------
# File validation
#---------------------------------------------

def parseFileBegin(data, startpos):
    found = FILEBEGIN_RE.search(data, startpos)
    if found != None:
        result = {}
        result["filetype"]    = found.group("ft")
        result["filename"]    = found.group("fn")
        result["filesize"]    = int(found.group("fsize"))
        result["compression"] = found.group("comp")
        result["cfilesize"]   = -1
        if found.group("csize") != "":
            result["cfilesize"] = int(found.group("csize"))
        result["blocksize"]   = int(found.group("bsize"))
        result["offset"]      = int(found.group("offset"))
        result["md5file"]     = found.group("md5")
        result["md5cfile"]    = found.group("cmd5")
        result["tagstart"]    = found.start()
        result["tagend"]      = found.end()
        result["filestart"]   = result["tagend"] + result["offset"]
        size = result["cfilesize"]
        if size == -1:
            size = result["filesize"]
        result["fileend"]     = result["filestart"] + size
        result["blockend"]    = result["tagend"] + result["blocksize"]
        
        return result
    else:
        return None


def extractFile(data, tagInfo, toDisk, dir):
    result = {"writeSuccessful": False, "correctMD5": False,
            "correctCMD5": False, "decompressionSuccessful": False}
    start = tagInfo["filestart"]
    end = tagInfo["fileend"]
    
    compressedFile = data[start:end]
    if tagInfo["compression"] != "none":
        hexmd5 = checksums.md5(compressedFile).hexdigest()
        if hexmd5 != tagInfo["md5cfile"]:
            logger.error("compressed md5 incorrect")
        else:
            logger.log(4, "compressed md5 correct")
            result["correctCMD5"] = True
    else:
        if tagInfo["md5cfile"] != "":
            logger.error("compression == none but compressed md5 not empty")
        else:
            result["correctCMD5"] = True
    
    decompressedFile, partResult = decompressFile(compressedFile, tagInfo)
    result.update(partResult)
    
    if decompressedFile != None and toDisk:
        result["writeSuccessful"] = writeToDisk(decompressedFile, tagInfo, dir)
    elif toDisk == False:
        result["writeSuccessful"] = True
    return result

def decompressFile(filebody, tagInfo):
    result = {"decompressionSuccessful": False, "correctMD5": False,
            "md5": "", "size": -1}
    
    compr = tagInfo["compression"]
    logger.log(4, "compression is %s", compr)
    if compr == "none":
        decompressedFile = filebody
        result["decompressionSuccessful"] = True
    elif compr == "flate":
        try:
            decompressedFile = zlib.decompress(filebody)
            result["decompressionSuccessful"] = True
        except Exception, e:
            logger.error("decompression failed")
            return None, result
    
    hexmd5 = checksums.md5(decompressedFile).hexdigest()
    result["md5"] = hexmd5
    result["size"] = tagInfo["filesize"]
    if hexmd5 != tagInfo["md5file"]:
        logger.error("uncompressed md5 incorrect")
        return None, result
    else:
        logger.log(4, "uncompressed md5 correct")
        result["correctMD5"] = True
        return decompressedFile, result


def writeToDisk(filebody, tagInfo, destDir):
    filename = "%(index)d_%(filename)s" % tagInfo
    filename = os.path.join(destDir, filename)
    try:
        logger.log(4, "writing file %s to %s", tagInfo["filename"], filename)
        outfile = file(filename, "wb")
        outfile.write(filebody)
        outfile.close()
    except Exception, e:
        logger.error("could not write file: %s", e)
        return False
    return True


def validateFile(data, tagInfo):
    logger.log(3, "file: <%(index)s> '%(filename)s'" % tagInfo)
    logger.log(3, "filetype: '%(filetype)s'" % tagInfo)

