
"""
This module contains classes to represent workflow files in a VHIST workflow
step. Workflow files can represent any type of data used in or created by
a workflow step, e.g. the source code of your python script or program,
important config files, input data, generated outputs, log files or other
VHIST files. We categorize workflow files into two classes: input files and
output files. Input files are all types of files used as input into the
workflow. Output files are files generated by your script or program.

You can embed workflow files directly into a VHIST file. This is especially
useful for small files, such as logfiles, configuration files or scripts.
Larger files are usually only mentioned in the VHIST file (with information
about the file size and md5 checksum), as large files inflate the VHIST file's
size immensely.

================          ===========
Attribute Name            Description
================          ===========
filename                  The name of the file (relative or absolute).
purpose                   States if the file is an input file or an output
                          file. This attribute excepts a %FilePurpose
                          object.
description               A description of the file's role within the workflow
                          step.
comment                   An additional comment about the file.
filetype                  The type of the file in the form of a MIME type or
                          MIME type-like descriptor. Examples are
                          ``images/png`` or ``binary/ecat7``. If you do not
                          specify a filetype (set ``filetype`` to ``None``),
                          pyvhist will automatically try to derive a mimetype
                          based on the files extension.
md5file                   A file, which contains the md5 checksum of a file.
                          Useful only if you do not embed the workflow file
                          inside the VHIST file and already calculated the
                          checksum of the file for some reason.
isPreviewWs               Either ``True`` or ``False``. If set to ``True``,
                          the file will be displayed as a preview image for
                          the workflow step. Do not use this option directly.
                          Instead use the ``previews`` attribute of
                          %%vhistsection.VhistSection.
isPreview                 Either ``True`` or ``False``. If set to ``True``,
                          the file will be displayed as a preview image for
                          the latest preceeding file, which was not a preview
                          itself. Do not use this option directly.
                          Instead use the ``previews`` attribute of %VhistFile.
isThumbnail               Do not use this option.
isThumbnailOnly           Do not use this option.
optional                  Constitutes if the file is optional or not. Set this
                          option to ``True`` if the file might not exist.
automd5                   Either ``True`` or ``False``. You can set this option
                          to ``False`` if you do not embed this file and do not
                          want VHIST to calculate a checksum for the file (for
                          example because the file is too big or you supplied
                          your own md5 sum with the ``md5file`` option.
embed                     Either ``True`` or ``False``. If this option is
                          ``True``, the file will be embedded into the VHIST
                          file. Otherwise, the file will only be mentioned in
                          the VHIST file.
compress                  Either ``True`` or ``False``. If ``compress`` is
                          set to ``True``, the file will be compressed using
                          the flate compression method and embedded in
                          compressed form.
================          ===========
"""

import types

from external.ordereddict.ordereddict import OrderedDict
from typecheck import typeCheck
from limitedsetter import limitedSetter
import getmimetype
import userattrs
import jsonify

class FilePurpose(object):
    """ The purpose of the file. Either input or output """
    def __init__(self, purpose):
        self._purpose = purpose
    
    @staticmethod
    def InFile():
        return FilePurpose("in")
    
    @staticmethod
    def OutFile():
        return FilePurpose("out")

    def isInfile(self):
        return self._purpose == "in"

    def isOutfile(self):
        return self._purpose == "out"


def infile():
    return FilePurpose("in")


def outfile():
    return FilePurpose("out")


def InFile(filename, description = None, comment = None, filetype = None,
        md5file = None, isPreviewWs = None, isPreview = None,
        isThumbnail = None, isThumbnailOnly = None, optional = None,
        automd5 = None, embed = None, compress = None):
    """ Creates the VhistFile object with purpose set to `Input File`.
    All other arguments behave just like the arguments of the
    %VhistFile constructor.
    """
    return VhistFile(filename, infile(), description, comment, filetype,
            md5file, isPreviewWs, isPreview, isThumbnail, isThumbnailOnly,
            optional, automd5, embed, compress)


def OutFile(filename, description = None, comment = None, filetype = None,
        md5file = None, isPreviewWs = None, isPreview = None,
        isThumbnail = None, isThumbnailOnly = None, optional = None,
        automd5 = None, embed = None, compress = None):
    """ Creates the VhistFile object with purpose set to `Output File`.
    All other arguments behave just like the arguments of the
    %VhistFile constructor.
    """
    return VhistFile(filename, outfile(), description, comment, filetype,
            md5file, isPreviewWs, isPreview, isThumbnail, isThumbnailOnly,
            optional, automd5, embed, compress)


class VhistFile(object):
    """ This class represents a vhist workflow file. Each instance of this
    class can be included in one workflow step (see
    %%vhistsection.VhistSection).
    """
    __slots__ = ["_filename", "_purpose", "_description", "_comment", 
            "_filetype", "_md5file", "_isPreviewWs", "_isPreview",
            "_isThumbnail", "_isThumbnailOnly", "_optional", "_automd5",
            "_embed", "_compress", "_userAttrs"]
    def __init__(self, filename, purpose, description = None, comment = None,
            filetype = None, md5file = None, isPreviewWs = None,
            isPreview = None, isThumbnail = None, isThumbnailOnly = None,
            optional = None, automd5 = None, embed = None, compress = None):
        
        # string properties
        self.filename = filename
        self.purpose = purpose
        self.description = description
        self.comment = comment
        
        if filetype == None:
            self.filetype = getmimetype.getMimeType(self.filename)
        else:
            self.filetype = filetype
        
        self.md5file = md5file
        
        # boolean properties
        self.isPreviewWs = isPreviewWs
        self.isPreview = isPreview
        self.isThumbnail = isThumbnail
        self.isThumbnailOnly = isThumbnailOnly
        self.optional = optional
        self.automd5 = automd5
        self.embed = embed
        self.compress = compress
        
        # user-defined attributes
        self._userAttrs = OrderedDict()
        #self.noNewMembers = True
    

    def getFilename(self):
        return self._filename
    
    def setFilename(self, filename):
        typeCheck("filename", filename, basestring)
        self._filename = filename
    
    def getPurpose(self):
        return self._purpose
    
    def setPurpose(self, purpose):
        typeCheck("purpose", purpose, FilePurpose)
        self._purpose = purpose
    
    def getDescription(self):
        return self._description
    
    def setDescription(self, description):
        typeCheck("description", description, (basestring, types.NoneType))
        self._description = description
    
    def getComment(self):
        return self._comment
    
    def setComment(self, comment):
        typeCheck("comment", comment, (basestring, types.NoneType))
        self._comment = comment
    
    def getFiletype(self):
        return self._filetype
    
    def setFiletype(self, filetype):
        typeCheck("filetype", filetype, (basestring, types.NoneType))
        self._filetype = filetype
    
    def getMd5File(self):
        return self._md5file
    
    def setMd5File(self, md5file):
        typeCheck("md5file", md5file, (basestring, types.NoneType))
        self._md5file = md5file
    
    def getIsPreviewWs(self):
        return self._isPreviewWs
    
    def setIsPreviewWs(self, isPreviewWs):
        typeCheck("isPreviewWs", isPreviewWs, (bool, types.NoneType))
        self._isPreviewWs = isPreviewWs

    def getIsPreview(self):
        return self._isPreview
    
    def setIsPreview(self, isPreview):
        typeCheck("isPreview", isPreview, (bool, types.NoneType))
        self._isPreview = isPreview
    
    def getIsThumbnail(self):
        return self._isThumbnail
    
    def setIsThumbnail(self, isThumbnail):
        typeCheck("isThumbnail", isThumbnail, (bool, types.NoneType))
        self._isThumbnail = isThumbnail
    
    def getIsThumbnailOnly(self):
        return self._isThumbnailOnly
    
    def setIsThumbnailOnly(self, isThumbnailOnly):
        typeCheck("isThumbnailOnly", isThumbnailOnly, (bool, types.NoneType))
        self._isThumbnailOnly = isThumbnailOnly
    
    def getOptional(self):
        return self._optional
    
    def setOptional(self, optional):
        typeCheck("optional", optional, (bool, types.NoneType))
        self._optional = optional
    
    def getAutoMd5(self):
        return self._automd5
    
    def setAutoMd5(self, automd5):
        typeCheck("automd5", automd5, (bool, types.NoneType))
        self._automd5 = automd5
    
    def getEmbed(self):
        return self._embed
    
    def setEmbed(self, embed):
        typeCheck("embed", embed, (bool, types.NoneType))
        self._embed = embed
    
    def getCompress(self):
        return self._compress
    
    def setCompress(self, compress):
        typeCheck("compress", compress, (bool, types.NoneType))
        self._compress = compress
    
    def getUserAttrs(self):
        return self._userAttrs
    
    def isInfile(self):
        """ Returns whether the file is an input file or not. """
        return self.purpose.isInfile()
    
    def isOutfile(self):
        """ Returns whether the file is an output file or not. """
        return self.purpose.isOutfile()
    
    filename = property(getFilename, setFilename,
            doc="The name of the file, including the path")
    purpose = property(getPurpose, setPurpose,
            doc="Whether the file is an input or output file")
    description = property(getDescription, setDescription,
            doc="The description of the file")
    comment = property(getComment, setComment,
            doc="The comment of the file")
    filetype = property(getFiletype, setFiletype,
            doc="The filetype of the file")
    md5file = property(getMd5File, setMd5File,
            doc="The md5 file of the file")
    isPreviewWs = property(getIsPreviewWs, setIsPreviewWs,
            doc="The isPreviewWs property of the file")
    isPreview = property(getIsPreview, setIsPreview,
            doc="The isPreview property of the file")
    isThumbnail = property(getIsThumbnail, setIsThumbnail,
            doc="The isThumbnail property of the file")
    isThumbnailOnly = property(getIsThumbnailOnly, setIsThumbnailOnly,
            doc="The isThumbnailOnly property of the file")
    optional = property(getOptional, setOptional,
            doc="The optional property of the file")
    automd5 = property(getAutoMd5, setAutoMd5,
            doc="The automd5 property of the file")
    embed = property(getEmbed, setEmbed,
            doc="The embed property of the file")
    compress = property(getCompress, setCompress,
            doc="The compress property of the file")
    userAttrs = property(getUserAttrs, None,
            doc="A dictionary of key-value pairs")
    
    def assembleCommandline(self):
        def boolToFlag(flagname, boolean):
            if boolean:
                return flagname
            else:
                return "no-%s" % flagname
        
        commandline = []
        if self.purpose.isInfile():
            commandline.extend(["-i", self.filename])
        elif self.purpose.isOutfile():
            commandline.extend(["-o", self.filename])
        else:
            raise RuntimeError("File is neither an input file, nor an output file")
        if self.description != None:
            commandline.extend(["-a", "description", self.description])
        if self.comment != None:
            commandline.extend(["-a", "comment", self.comment])
        if self.filetype != None:
            commandline.extend(["-a", "filetype", self.filetype])
        if self.md5file != None:
            commandline.extend(["-a", "md5file", self.md5file])
        if self.isPreviewWs != None:
            commandline.extend(["-f", boolToFlag("previewws", self.isPreviewWs)])
        if self.isPreview != None:
            commandline.extend(["-f", boolToFlag("preview", self.isPreview)])
        if self.isThumbnail != None:
            commandline.extend(["-f", boolToFlag("thumbnail", self.isThumbnail)])
        if self.isThumbnailOnly != None:
            commandline.extend(["-f", boolToFlag("thumbnailonly",
                    self.isThumbnailOnly)])
        if self.optional != None:
            commandline.extend(["-f", boolToFlag("optional", self.optional)])
        if self.automd5 != None:
            commandline.extend(["-f", boolToFlag("automd5", self.automd5)])
        if self.embed != None:
            commandline.extend(["-f", boolToFlag("embed", self.embed)])
        if self.compress != None:
            commandline.extend(["-f", boolToFlag("compress", self.compress)])
        
        for key, value in self._userAttrs.items():
            strValue = jsonify.jsonify(value)
            if isinstance(value, userattrs.UserValue) and value.unit \
                    is not None:
                commandline.extend(["-w", key, strValue, value.unit])
            else:
                commandline.extend(["-u", key, strValue])
        
        return commandline

