
import os
import fnmatch
import baseplugin

MB = 1024 ** 2
WATER_MARK1_MB = 3.5
WATER_MARK2_MB = 6
WATER_MARK1 = WATER_MARK1_MB * MB
WATER_MARK2 = WATER_MARK2_MB * MB


def splitPatterns(patterns):
    patternsList = patterns.split(",")
    patternsList = [p.strip() for p in patternsList]
    patternsList = [p for p in patternsList if p != ""]
    return patternsList


class EmbedPlugin(baseplugin.BasePlugin):
    """
    Decides for each file whether or not the file should be embedded into the
    generated VHIST files. Prevents VHIST files from getting very large.
    """
    def __init__(self):
        self.embedPatterns = os.environ.get("VHISTIFY_EMBED_PATTERNS", "")
        self.noEmbedPatterns = os.environ.get("VHISTIFY_DONT_EMBED_PATTERNS",
                "")
        
        self.embedPatterns = splitPatterns(self.embedPatterns)
        self.noEmbedPatterns = splitPatterns(self.noEmbedPatterns)
    
    def forcedEmbed(self, filename):
        """ returns true if the file matches the VHISTIFY_EMBED_PATTERNS
            variable. """
        return any(fnmatch.fnmatch(filename, pat) for pat in
                self.embedPatterns)
    
    def forcedDontEmbed(self, filename):
        """ returns true if the file matches the VHISTIFY_DONT_EMBED_PATTERNS
            variable but not VHISTIFY_EMBED_PATTERNS. """
        if not self.forcedEmbed(filename):
            return any(fnmatch.fnmatch(filename, pat) for pat in
                    self.noEmbedPatterns)
    
    def convertFiles(self, vfiles):
        """ Sets the embed property for all files. vhist files are always
            included! """
        filesizes = []
        totalSize = 0
        
        for vfile in vfiles:
            filename = vfile.filename
            if os.path.isfile(filename):
                filesize = os.path.getsize(filename)
                if vfile.embed == True and not filename.endswith(".vhist"):
                    totalSize += filesize
                elif vfile.embed is None:
                    filesizes.append((vfile, filesize))
        
        filesizes = sorted(filesizes, key=lambda x: x[1])
        
        i = 0
        while totalSize < WATER_MARK2 and i < len(filesizes):
            vfile, filesize = filesizes[i]
            if self.forcedEmbed(vfile.filename):
                vfile.embed = True
                totalSize += filesize
            elif self.forcedDontEmbed(vfile.filename):
                vfile.embed = False
            elif totalSize + filesize < WATER_MARK2 and filesize < WATER_MARK1:
                vfile.embed = True
                totalSize += filesize
            else:
                vfile.embed = False
            i += 1
        
        return vfiles
    
    @classmethod
    def getSupportedEnvironmentVariables(cls):
        return ["VHISTIFY_EMBED_PATTERNS", "VHISTIFY_DONT_EMBED_PATTERNS"]
    
    @classmethod
    def getDocumentation(cls):
        return DOCUMENTATION


DOCUMENTATION = """
The "embed" plugin iterates over all accessed and written files and decides
whether or not each file should be embed into the generated VHIST files.

The plugin will only embed files, which are smaller than %.1f MB. It will
also make sure that the sum of all filesizes will be smaller than %.1f MB.
Smaller files are embedded first.

Keep in mind that these thresholds
refer to the sizes of uncompressed files. As files are usually stored in
compressed form inside a VHIST file, the VHIST file's actual size will differ.
Also keep in mind that other plugins can override the settings set by
the embed plugin.

You can adjust the behaviour of this plugin with the two environment variables

VHISTIFY_EMBED_PATTERNS and VHISTIFY_DONT_EMBED_PATTERNS

Each takes a comma-separated list of wildcard patterns (e.g.
"*.txt,output.*"). If a file matches a pattern in one of these lists, it
is always embedded or never embedded. If a file matches patterns in both lists,
the file is embedded. These decisions may also be overridden by other plugins.

""" % (WATER_MARK1_MB, WATER_MARK2_MB)

