
import os
import sys
import baseplugin

def splitPath(path):
    path = os.path.expanduser(path)
    path = os.path.realpath(path)
    if sys.platform in ["linux", "linux2", "darwin", "sunos"]:
        curPath = path
        pathList = []
        while curPath != "" and curPath != "/":
            curPath, lastPath = os.path.split(curPath)
            pathList.insert(0, lastPath)
        return pathList
    else:
        raise NotImplementedError("Not supported on Windows, yet")


def minid(x, y):
    if x <= y:
        return x, 0
    else:
        return y, 1


def moreSpecificThan(path1, path2):
    """ checks if path1 is more specific than path2. Creates a partial order
    on paths
    - /foo/bar is more specific than /foo
    - /a is not more specific than /b, /b is not more specific than /a
    - /a is not more specific than /a
    """
    minLength, minId = minid(len(path1), len(path2))
    # minId == 1 => path1 is longer than path2
    return path1[:minLength] == path2[:minLength] and minId == 1


def matches(filename, splitDir):
    splitFile = splitPath(filename)
    return moreSpecificThan(splitFile, splitDir)


def orderPaths(paths):
    orderedPaths = []
    
    for (path, mode) in paths:
        path = splitPath(path)
        pathTuple = (path, mode)
        positionFound = False
        for i, (entry, mode2) in enumerate(orderedPaths):
            if moreSpecificThan(path, entry):
                positionFound = True
                orderedPaths.insert(i, pathTuple)
                break
            elif path == entry:
                if mode == mode2:
                    positionFound = True
                    break
                else:
                    dirname = "/" + os.path.join(*path)
                    raise RuntimeError("directory '%s' is in "
                            "$VHISTIFY_EXCLUDE_DIRS and "
                            "$VHISTIFY_INCLUDE_DIRS" % dirname)
        if not positionFound:
            orderedPaths.append(pathTuple)
            
    return orderedPaths


def getDirsList(envvar):
    paths = os.environ.get(envvar, "")
    pathList = paths.split(os.pathsep)
    pathList = [p for p in pathList if p != ""]
    return pathList



class EnvPlugin(baseplugin.BasePlugin):
    """ Read the environment variables VHISTIFY_EXCLUDE_DIRS and
        VHISTIFY_INCLUDE_DIRS and considers them when including/excluding
        files into/from the generated VHIST files.
    """
    
    def __init__(self):
        baseplugin.BasePlugin.__init__(self)
        self.excludeDirs = getDirsList("VHISTIFY_EXCLUDE_DIRS")
        self.excludeDirs = [(d, "exclude") for d in self.excludeDirs]
        
        self.amendDirs = getDirsList("VHISTIFY_INCLUDE_DIRS")
        self.amendDirs = [(d, "include") for d in self.amendDirs]
        
        self.partialOrder = orderPaths(self.excludeDirs + self.amendDirs)
        
        #self.excludePatterns = getPatternsList("VHISTIFY_EXCLUDE_PATTERNS")
        #self.amendPatterns = getPatternsList("VHISTIFY_INCLUDE_PATTERNS")
    
    def filterFiles(self, files):
        outFiles = []
        for f in files:
            found = False
            for path, mode in self.partialOrder:
                if matches(f.filename, path):
                    found = True
                    if mode == "exclude":
                        break
                    elif mode == "include":
                        outFiles.append(f)
                        break
                    else:
                        raise RuntimeError("code should not reach this point")
            if not found:
                outFiles.append(f)
        
        return outFiles
    
    
    def amendFiles(self, files):
        outFiles = []
        for f in files:
            for path, mode in self.partialOrder:
                if matches(f.filename, path):
                    if mode == "exclude":
                        break
                    elif mode == "include":
                        outFiles.append(f)
                        break
                    else:
                        raise RuntimeError("code should not reach this point")
        
        return outFiles
    
    @classmethod
    def getSupportedEnvironmentVariables(cls):
        return ["VHISTIFY_EXCLUDE_DIRS", "VHISTIFY_INCLUDE_DIRS"]
    
    @classmethod
    def getDocumentation(cls):
        return DOCUMENTATION

DOCUMENTATION = """
The "env" plugin allows you to customize which files are included into
or excluded from the generated VHIST files.

The plugin reads the environment variables VHISTIFY_INCLUDE_DIRS
and VHISTIFY_EXCLUDE_DIRS, which - if defined - should contain a colon (:)
separated list of directories (just like the PATH environment variable on
linux/unix).

The paths in these lists are first converted into absolute
paths with all symbolic links resolved. The plugin then checks all
files used by your application. If the file (again after resolving symbolic
links) is part of a directory from the exclude list, it is removed from the
generated VHIST files. If it is part of a directory from the include list,
it is kept in the VHIST files. If it matches several paths from both lists,
the most specific entry wins, e.g. /data/references is more specific than
/data/.

Might not work correctly if you access the same network share from via
different filesystem mounts. It might also not work as expected if you use 
symbolic links to files or subdirectory (e.g. you want to exclude /data but
/data/mydata is in fact a symbolic link to /remote-data/mydata and is
therefore still included into the VHIST files).
"""

