
import os
import baseplugin
import zipsource

MB = 1024 * 1024

def isMatlabStartupFile(filename):
    basename = os.path.basename(filename)
    return basename == "startup.m" or basename == "pathdef.m"


def getVersionFile(matlabFile):
    # matlabFile must be ispc.m, matlab.env or usejava.m (in the general/ dir)
    dirname = os.path.dirname(matlabFile)
    version1 = os.path.join(dirname, "Contents.txt")
    version2 = os.path.join(dirname, "Contents.m")

    if os.path.isfile(version1):
        return version1
    elif os.path.isfile(version2):
        return version2
    else:
        return None


def readVersion(filename):
    with open(filename, "rb") as f:
        l1 = f.readline()
        l2 = f.readline().strip()
        if l2.startswith("%"):
            l2 = l2[1:].strip()
        return l2


def isExecutableFile(filename):
    return os.path.isfile(filename) and os.access(filename, os.X_OK)


def getVersionFiles(syscalls):
    openCalls = [s for s in syscalls if s.name == "open" ]
    files = [s.args[0] for s in openCalls]
    envFiles = [f for f in files if f.endswith("general/matlab.env") or
            f.endswith("general/ispc.m") or f.endswith("usejava.m")]
    versionFiles = [getVersionFile(f) for f in envFiles]
    versionFiles = [f for f in versionFiles if f is not None]
    versionFiles = list(set(versionFiles))
    return versionFiles


def getVersion(syscalls):
    versionFiles = getVersionFiles(syscalls)
    return [readVersion(f) for f in versionFiles]


def getMatlabRootDirs(syscalls):
    # the (correct) version file is in $MATLABROOT/toolbox/matlab/general
    # check if $MATLABROOT/bin/matlab exists, then return $MATLABROOT
    versionFiles = getVersionFiles(syscalls)
    rootDirs = []
    for f in versionFiles:
        versionDir = os.path.dirname(f)
        binDir = os.path.abspath(os.path.join(versionDir, "../../../bin"))
        matlabBin = os.path.join(binDir, "matlab")
        if isExecutableFile(matlabBin):
            rootDir = os.path.abspath(os.path.join(binDir, ".."))
            rootDir = os.path.realpath(rootDir)
            if not rootDir.endswith("/"):
                rootDir = rootDir + "/"
            rootDirs.append(rootDir)
    return rootDirs


class MatlabPlugin(baseplugin.BasePlugin):
    """
    Enhances the output of Matlab jobs.
    """
    
    def convertFiles(self, files):
        matlabFiles = [f for f in files if f.filename.endswith(".m")]
        normalFiles = [f for f in files if not f.filename.endswith(".m")]
        
        zipfilename = self.tmpdir.createTempFile("sourcecode.zip")
        zipsource.zipSource(matlabFiles, zipfilename)
        vzip = self.vhistfileModule.InFile(zipfilename,
                description="matlab sourcecode", embed=True)
        
        for f in normalFiles:
            fn = f.filename
            if (fn.endswith("vhistify-inputs.mat") or
                    fn.endswith("vhistify-outputs.mat")):
                if os.path.getsize(fn) < 2 * MB:
                    f.embed = True # otherwise, let embed.py decide

        return [vzip] + normalFiles
    
    def filterFiles(self, files):
        matlabRoots = getMatlabRootDirs(self.infoObj.calls)
        outFiles = []
        for f in files:
            if all(not f.filename.startswith(root) for root in matlabRoots):
                outFiles.append(f)
        return outFiles
    
    def amendFiles(self, files):
        amendedFiles = [f for f in files if isMatlabStartupFile(f.filename) or
                f.filename.endswith("vhistify-inputs.mat") or
                f.filename.endswith("vhistify-outputs.mat")]
        return amendedFiles
    
    def updateSection(self, section):
        matlabExecutables = []
        for realProgram, program in self.infoObj.programs:
            if "matlab" in os.path.basename(realProgram):
                matlabExecutables.append(realProgram)
        
        if len(matlabExecutables) > 0:
            section.userAttrs["matlabExecutables"] = matlabExecutables
        else:
            section.userAttrs["matlabExecutables"] = \
                    "could not detect matlab executable"
        
        # try to detect matlab version
        matlabVersions = getVersion(self.infoObj.calls)
        
        if len(matlabVersions) > 0:
            section.userAttrs["matlabVersions"] = matlabVersions
        else:
            section.userAttrs["matlabVersions"] = \
                    "could not detect matlab version"
        
        return section
    
    @classmethod
    def getDocumentation(cls):
        return DOCUMENTATION


DOCUMENTATION = """
The Matlab plugin enhances the generated VHIST files for Matlab jobs.
If you enable this plugin, vhistify will try to detect the Matlab
executable as well as the version of Matlab and document this information
in the generated VHIST files. The plugin will also exclude all files from
the Matlab installation (such as Matlab's sourcecode) from the VHIST file.

Moreover, all read files with the extension .m are packed into a zip
archive named sourcecode.zip.
"""

