
import os
from fnmatch import fnmatch

import baseplugin

def getVersionFile(spmFile):
    dirname = os.path.dirname(spmFile)
    version1 = os.path.join(dirname, "Contents.txt")
    version2 = os.path.join(dirname, "Contents.m")
    
    if os.path.isfile(version1):
        return version1
    elif os.path.isfile(version2):
        return version2
    else:
        return None


def readVersion(filename):
    with open(filename, "rb") as f:
        l1 = f.readline()
        l2 = f.readline().strip()
        if l2.startswith("%"):
            l2 = l2[1:].strip()
        return l2


def isPartOfSPM(filename, spmDirs):
    return any(filename.startswith(d) for d in spmDirs)


def isSpmSourceFile(filename):
    dirname, basename = os.path.split(filename)
    if fnmatch(basename, "spm_*.m") or fnmatch(basename, "spm.m"):
        if os.path.isfile(getSpmMain(filename)):
            return True
    return False


def getSpmMain(filename):
    dirname, basename = os.path.split(filename)
    return os.path.join(dirname, "spm.m")


def unique(alist):
    # returns unique entries and keeps order of items
    outlist = []
    for item in alist:
        if item not in outlist:
            outlist.append(item)
    return outlist


def isSPMTemplate(filename, spmDirs):
    """ tests if the file is some SPM template or apriori data """
    # all files in these directories should be included in the VHIST files
    # dir names should include the / at the end
    dirsToInclude = ["apriori/", "canonical/", "EEGtemplates/", "templates/",
            "tpm/"]
    # all files with this extension should be included in the VHIST files
    extensionsToInclude = [".nii", ".nii.gz"]
    for spmDir in spmDirs:
        for subdir in dirsToInclude:
            fulldir = os.path.join(spmDir, subdir)
            if filename.startswith(fulldir):
                return True
        
        if filename.startswith(spmDir):
            for ext in extensionsToInclude:
                if filename.endswith(ext):
                    return True
    return False


class SPMPlugin(baseplugin.BasePlugin):
    """
    Enhances the output of SPM jobs.
    """
    def __init__(self):
        self.spmMains = []
        self.spmVersions = []
        self.spmDirs = []
    
    def filterFiles(self, files):
        # find .m files that are probably part of spm
        partOfSpm = [f.filename for f in files if
                isSpmSourceFile(f.filename)]
        
        # find spm.m files
        self.spmMains = [getSpmMain(f) for f in partOfSpm]
        self.spmMains = unique(self.spmMains)
        
        # find version files Contents.*
        versionFiles = [getVersionFile(f) for f in self.spmMains]
        versionFiles = [f for f in versionFiles if f is not None]
        
        # read version strings from version files
        self.spmVersions = [readVersion(f) for f in versionFiles]
        
        # exclude all spm files from spm the generated VHIST files
        self.spmDirs = [os.path.realpath(os.path.dirname(f)) for f in
                versionFiles]
        
        return [f for f in files if not isPartOfSPM(f.filename, self.spmDirs)]
    
    def amendFiles(self, files):
        return [f for f in files if isSPMTemplate(f.filename, self.spmDirs)]
    
    def convertFiles(self, files):
        for f in files:
            if isSPMTemplate(f.filename, self.spmDirs):
                f.embed = False
        return files
    
    def updateSection(self, section):
        if len(self.spmMains) > 0:
            section.userAttrs["spmMains"] = self.spmMains
        else:
            section.userAttrs["spmMains"] = "could not detect SPM main file"
        
        if len(self.spmVersions) > 0:
            section.userAttrs["spmVersions"] = self.spmVersions
        else:
            section.userAttrs["spmVersions"] = "could not detect SPM version"
        
        return section
    
    @classmethod
    def getDocumentation(cls):
        return DOCUMENTATION


DOCUMENTATION = """
The SPM plugin enhances the generated VHIST files for SPM jobs. It detects
the version of SPM used by your job and documents the location of the
spm.m file of your SPM installation. The script will also exclude files
that are part of the SPM installation from the generated VHIST files (with
the exception of templates and apriori files, which are listed but not
embedded in the VHIST files). The plugin requires that the spm function is
called at least once (usually happens at some point if you use any part of
SPM).
"""

